Package: gtable
Title: Arrange 'Grobs' in Tables
Version: 0.3.6
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut"),
    person("Thomas Lin", "Pedersen", , "thomas.pedersen@posit.co", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Tools to make it easier to work with "tables" of 'grobs'. The
    'gtable' package defines a 'gtable' grob class that specifies a grid
    along with a list of grobs and their placement in the grid. Further
    the package makes it easy to manipulate and combine 'gtable' objects
    so that complex compositions can be built up sequentially.
License: MIT + file LICENSE
URL: https://gtable.r-lib.org, https://github.com/r-lib/gtable
BugReports: https://github.com/r-lib/gtable/issues
Depends: R (>= 4.0)
Imports: cli, glue, grid, lifecycle, rlang (>= 1.1.0), stats
Suggests: covr, ggplot2, knitr, profvis, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2024-10-25
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2024-10-25 12:42:05 UTC; thomas
Author: Hadley Wickham [aut],
  Thomas Lin Pedersen [aut, cre],
  Posit Software, PBC [cph, fnd]
Maintainer: Thomas Lin Pedersen <thomas.pedersen@posit.co>
Repository: CRAN
Date/Publication: 2024-10-25 13:20:02 UTC

# Generated by roxygen2: do not edit by hand

S3method("[",gtable)
S3method("dimnames<-",gtable)
S3method(as.gtable,default)
S3method(as.gtable,grob)
S3method(as.gtable,gtable)
S3method(cbind,gtable)
S3method(dim,gtable)
S3method(dimnames,gtable)
S3method(heightDetails,gtable)
S3method(length,gtable)
S3method(makeContent,gtable)
S3method(makeContext,gtable)
S3method(plot,gtable)
S3method(print,gtable)
S3method(rbind,gtable)
S3method(t,gtable)
S3method(widthDetails,gtable)
export(as.gtable)
export(gtable)
export(gtable_add_col_space)
export(gtable_add_cols)
export(gtable_add_grob)
export(gtable_add_padding)
export(gtable_add_row_space)
export(gtable_add_rows)
export(gtable_col)
export(gtable_col_spacer)
export(gtable_filter)
export(gtable_height)
export(gtable_matrix)
export(gtable_row)
export(gtable_row_spacer)
export(gtable_show_layout)
export(gtable_trim)
export(gtable_width)
export(is.gtable)
import(grid)
import(rlang)
importFrom(glue,glue)
importFrom(lifecycle,deprecated)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# gtable <a href="https://gtable.r-lib.org"><img src="man/figures/logo.png" align="right" height="138" /></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/r-lib/gtable/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/gtable/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/gtable)](https://CRAN.R-project.org/package=gtable)
[![Codecov test
coverage](https://codecov.io/gh/r-lib/gtable/graph/badge.svg)](https://app.codecov.io/gh/r-lib/gtable)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
<!-- badges: end -->

gtable is a layout engine built on top of the grid package. It is used
to abstract away the creation of (potentially nested) grids of viewports
into which graphic objects can be placed. gtable makes it easy to ensure
alignment of graphic elements and piecemeal compositions of complex
graphics. gtable is the layout engine powering
[ggplot2](https://ggplot2.tidyverse.org) and is thus used extensively by
many plotting functions in R without being called directly.

## Installation

You can install the released version of gtable from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("gtable")
```

or use the remotes package to install the development version from
[GitHub](https://github.com/r-lib/gtable)

``` r
# install.packages("pak")
pak::pak("r-lib/gtable")
```

## Example

ggplot2 uses gtable for laying out plots, and it is possible to access
the gtable representation of a plot for inspection and modification:

``` r
library(gtable)
library(ggplot2)

p <- ggplot(mtcars, aes(mpg, disp)) + geom_point()

p_table <- ggplotGrob(p)

p_table
#> TableGrob (16 x 13) "layout": 22 grobs
#>     z         cells             name                                         grob
#> 1   0 ( 1-16, 1-13)       background               rect[plot.background..rect.38]
#> 2   5 ( 8- 8, 6- 6)           spacer                               zeroGrob[NULL]
#> 3   7 ( 9- 9, 6- 6)           axis-l           absoluteGrob[GRID.absoluteGrob.26]
#> 4   3 (10-10, 6- 6)           spacer                               zeroGrob[NULL]
#> 5   6 ( 8- 8, 7- 7)           axis-t                               zeroGrob[NULL]
#> 6   1 ( 9- 9, 7- 7)            panel                      gTree[panel-1.gTree.17]
#> 7   9 (10-10, 7- 7)           axis-b           absoluteGrob[GRID.absoluteGrob.22]
#> 8   4 ( 8- 8, 8- 8)           spacer                               zeroGrob[NULL]
#> 9   8 ( 9- 9, 8- 8)           axis-r                               zeroGrob[NULL]
#> 10  2 (10-10, 8- 8)           spacer                               zeroGrob[NULL]
#> 11 10 ( 7- 7, 7- 7)           xlab-t                               zeroGrob[NULL]
#> 12 11 (11-11, 7- 7)           xlab-b titleGrob[axis.title.x.bottom..titleGrob.30]
#> 13 12 ( 9- 9, 5- 5)           ylab-l   titleGrob[axis.title.y.left..titleGrob.33]
#> 14 13 ( 9- 9, 9- 9)           ylab-r                               zeroGrob[NULL]
#> 15 14 ( 9- 9,11-11)  guide-box-right                               zeroGrob[NULL]
#> 16 15 ( 9- 9, 3- 3)   guide-box-left                               zeroGrob[NULL]
#> 17 16 (13-13, 7- 7) guide-box-bottom                               zeroGrob[NULL]
#> 18 17 ( 5- 5, 7- 7)    guide-box-top                               zeroGrob[NULL]
#> 19 18 ( 9- 9, 7- 7) guide-box-inside                               zeroGrob[NULL]
#> 20 19 ( 4- 4, 7- 7)         subtitle         zeroGrob[plot.subtitle..zeroGrob.35]
#> 21 20 ( 3- 3, 7- 7)            title            zeroGrob[plot.title..zeroGrob.34]
#> 22 21 (14-14, 7- 7)          caption          zeroGrob[plot.caption..zeroGrob.36]
```

A gtable object is a collection of graphic elements along with their
placement in the grid and the dimensions of the grid itself. Graphic
elements can span multiple rows and columns in the grid and be gtables
themselves allowing for very complex automatically arranging layouts.

A gtable object is itself a grob, and can thus be drawn using standard
functions from the grid package:

``` r
library(grid)
grid.draw(p_table) # alternative use plot(p_table)
```

<img src="man/figures/README-unnamed-chunk-3-1.png" alt="A scatterplot of miles per gallon against displacement based on the mtcars dataset. The plot is rendered by first converting the ggplot object to a gtable and then plotting the gtable" width="100%" />

While most people will interact with gtable through ggplot2, it is
possible to build a plot from the ground up.

``` r
# Construct some graphical elements using grid
points <- pointsGrob(
  x = runif(10),
  y = runif(10),
  size = unit(runif(10), 'cm')
)
xaxis <- xaxisGrob(at = c(0, 0.25, 0.5, 0.75, 1))
yaxis <- yaxisGrob(at = c(0, 0.25, 0.5, 0.75, 1))

# Setup the gtable layout
plot <- gtable(
  widths = unit(c(1.5, 0, 1, 0.5), c('cm', 'cm', 'null', 'cm')),
  heights = unit(c(0.5, 1, 0, 1), c('cm', 'null', 'cm', 'cm'))
)

# Add the grobs
plot <- gtable_add_grob(
  plot,
  grobs = list(points, xaxis, yaxis),
  t = c(2, 3, 2),
  l = c(3, 3, 2),
  clip = 'off'
)

# Plot
grid.draw(plot)
```

<img src="man/figures/README-unnamed-chunk-4-1.png" alt="A scatterplot of random numbers. The scatterplot is created by combining separate grobs created with grid using gtable." width="100%" />

#' Add a single grob, possibly spanning multiple rows or columns.
#' 
#' This only adds grobs into the table - it doesn't affect the table in 
#' any way.  In the gtable model, grobs always fill up the complete table
#' cell.  If you want custom justification you might need to 
#' 
#' @param x a \code{\link{gtable}} object
#' @param grobs a single grob or a list of grobs
#' @param t a numeric vector giving the top extent of the grobs
#' @param l a numeric vector giving the left extent of the grobs
#' @param b a numeric vector giving the bottom extent of the grobs
#' @param r a numeric vector giving the right extent of the grobs
#' @param z a numeric vector giving the order in which the grobs should be
#'   plotted.  Use \code{Inf} (the default) to plot above or \code{-Inf}
#'   below all existing grobs. By default positions are on the integers, 
#'  giving plenty of room to insert new grobs between existing grobs.
#' @param clip should drawing be clipped to the specified cells
#'   (\code{"on"}), the entire table (\code{"inherit"}), or not at all 
#'   (\code{"off"})
#' @param name name of the grob - used to modify the grob name before it's
#'   plotted.
#' @export
gtable_add_grob <- function(x, grobs, t, l, b = t, r = l, z = Inf, clip = "on", name = x$name) 
{
  stopifnot(is.gtable(x))
  if (is.grob(grobs)) grobs <- list(grobs)
  stopifnot(is.list(grobs))

  # Check that inputs have the right length
  if(!all(vapply(list(t, r, b, l, z, clip, name), len_same_or_1,
    logical(1), grobs))) {
    stop("Not all inputs have either length 1 or same length same as 'grobs'")
  }

  # If z is just one value, replicate to same length as grobs
  if (length(z) == 1) {
    z <- rep(z, length(grobs))
  }

  # Get the existing z values from x$layout, and new non-Inf z-values
  zval <- c(x$layout$z, z[!is.infinite(z)])
  if (length(zval) == 0) {
    # If there are no existing finite z values, set these so that
    # -Inf values get assigned ..., -2, -1, 0 and
    # +Inf values get assigned 1, 2, 3, ...
    zmin <- 1
    zmax <- 0
  } else {
    zmin <- min(zval)
    zmax <- max(zval)
  }
  z[z == -Inf] <- zmin - rev(seq_len(sum(z == -Inf)))
  z[z == Inf] <- zmax + seq_len(sum(z == Inf))

  t <- neg_to_pos(t, nrow(x))
  b <- neg_to_pos(b, nrow(x))
  l <- neg_to_pos(l, ncol(x))
  r <- neg_to_pos(r, ncol(x))
  
  layout <- data.frame(t = t, l = l, b = b, r = r, z = z, 
    clip = clip, name = name, 
    stringsAsFactors = FALSE)
  stopifnot(length(grobs) == nrow(layout))
  
  x$grobs <- c(x$grobs, grobs)
  x$layout <- rbind(x$layout, layout)
  
  x
}

context("Subsetting")

base <- gtable(unit(rep(1, 3), "null"), unit(rep(1, 3), "null"))
rownames(base) <- LETTERS[1:3]
colnames(base) <- letters[1:3]

test_that("dimensions correct after subsetting", {
  expect_equal(dim(base[, ]), c(3, 3))
  expect_equal(dim(base[1:3, 1:3]), c(3, 3))
  expect_equal(dim(base[T, T]), c(3, 3))
  expect_equal(dim(base[c("A", "B", "C"), c("a", "b", "c")]), c(3, 3))

  expect_equal(dim(base[1, 1]), c(1, 1))
  expect_equal(dim(base[c(T, F, F), c(T, F, F)]), c(1, 1))
  expect_equal(dim(base[-(2:3), -(2:3)]), c(1, 1))
  expect_equal(dim(base["A", "b"]), c(1, 1))

  expect_equal(dim(base[1:2, 2:3]), c(2, 2))
})

rect <- rectGrob()
mid <- gtable_add_grob(base, rect, 2, 2)
row <- gtable_add_grob(base, rect, 2, l = 1, r = 3)
col <- gtable_add_grob(base, rect, 2, t = 1, b = 3)

tlbr <- function(x) unname(unlist(x$layout[c("t", "l", "b", "r")]))

test_that("grobs moved to correct location", {

  expect_equal(tlbr(mid[2, 2]),     c(1, 1, 1, 1))
  expect_equal(tlbr(mid[2:3, 2:3]), c(1, 1, 1, 1))

  expect_equal(tlbr(mid[1:2, 1:2]), c(2, 2, 2, 2))
  expect_equal(tlbr(mid[1:3, 1:3]), c(2, 2, 2, 2))  
})

test_that("spanning grobs kept if ends kept", {
  
  expect_equal(length(row[, -2]), 1)
  expect_equal(tlbr(row[, -2]), c(2, 1, 2, 2))
  
  expect_equal(length(col[-2, ]), 1)
  expect_equal(tlbr(col[-2, ]), c(1, 2, 2, 2))

  expect_equal(length(row[, 1]), 0)
  expect_equal(length(col[1, ]), 0)
  
})


# Detailed tests for indexing with [.gtable ----------------------------------

# Some of these tests can be confusing; if you need to see
# what's going on, run grid.draw(gt)

# Make a bunch of grobs
g1 <- rectGrob()
g2 <- circleGrob()
g3 <- polygonGrob()
g4 <- linesGrob()
g5 <- circleGrob()
g6 <- rectGrob()

# Check that two gtable objects are the same.
# This allows for differences in how units are stored and other subtle
# changes that don't affect appearance.
equal_gtable <- function(a, b) {
  require(plyr)

  identical(a$grobs, b$grobs)  &&
  # Normalized z values are the same (ensuring same render order)
  # Also ignore row names
  all.equal(unrowname(z_normalise(a)$layout),
            unrowname(z_normalise(b)$layout)) &&
  # Test widths/heights for equality.
  # This is the best way I could think of, but it's not very nice
  all(convertUnit(a$widths  - b$widths,  "cm", valueOnly = TRUE) == 0) &&
  all(convertUnit(a$heights - b$heights, "cm", valueOnly = TRUE) == 0) &&
  all.equal(a$respect, b$respect) &&
  all.equal(a$rownames, b$rownames) &&
  all.equal(a$colnames, b$colnames)
}


# This will create a new gtable made with gtable_matrix
# using the specified cols and rows from grobmat.
# The sizes of the rows/cols are the same as the index values (but in cm)
make_gt <- function(grobmat, rows, cols) {
  gtable_matrix("test", grobmat[rows, cols, drop = FALSE],
    heights=unit(rows, "cm"), widths=unit(cols, "cm") )
}


test_that("Indexing with single-cell grobs", {
  # Make a 2x3 gtable where each cell has one grob
  grobmat <- matrix(list(g1, g2, g3, g4, g5, g6), nrow=2)
  gt <- make_gt(grobmat, 1:2, 1:3)

  # Indexing in ways that don't change gt
  expect_true(equal_gtable(gt, gt[1:2, 1:3]))
  expect_true(equal_gtable(gt, gt[]))
  expect_true(equal_gtable(gt, gt[1:2, ]))
  expect_true(equal_gtable(gt, gt[, 1:3]))

  # New table from contiguous cells
  expect_true(equal_gtable(gt[1, 1], make_gt(grobmat, 1, 1)))
  expect_true(equal_gtable(gt[2, 2], make_gt(grobmat, 2, 2)))
  expect_true(equal_gtable(gt[1:2, 1], make_gt(grobmat, 1:2, 1)))
  expect_true(equal_gtable(gt[1:2, 2], make_gt(grobmat, 1:2, 2)))
  expect_true(equal_gtable(gt[1, 1:3], make_gt(grobmat, 1, 1:3)))
  expect_true(equal_gtable(gt[1, 1:2], make_gt(grobmat, 1, 1:2)))
  expect_true(equal_gtable(gt[1:2, 1:2], make_gt(grobmat, 1:2, 1:2)))
  expect_true(equal_gtable(gt[1:2, 2:3], make_gt(grobmat, 1:2, 2:3)))

  # New table from non-contiguous cells
  expect_true(equal_gtable(gt[1, c(1, 3)], make_gt(grobmat, 1, c(1, 3))))
  expect_true(equal_gtable(gt[1:2, c(1, 3)], make_gt(grobmat, 1:2, c(1, 3))))
})


test_that("Indexing with names", {
  # Make a 2x3 gtable where each cell has one grob
  grobmat <- matrix(list(g1, g2, g3, g4, g5, g6), nrow=2)
  gt <- make_gt(grobmat, 1:2, 1:3)
  dimnames(gt) <- list(c("a","b"), c("x","y","z"))

  expect_true(equal_gtable(gt, gt[c("a","b"), c("x","y","z")]))
  expect_true(equal_gtable(gt[1, ], gt["a", ]))
  expect_true(equal_gtable(gt[, 2], gt[, "y"]))
  expect_true(equal_gtable(gt[, 2:3], gt[, c("y","z")]))
  expect_true(equal_gtable(gt[1, 1:2], gt["a", c("x","y")]))
  expect_true(equal_gtable(gt[1, 1:2], gt["a", 1:2]))
})



# Make a gtable with grobs that span cells
make_span_gt <- function(rows, cols) {
  # Make gtable with one grob at (1:1, 1:3) and another at (1:2, 1:2)
  gt <- gtable(name = "test",
    heights=unit(rows, "cm"), widths=unit(cols, "cm") )

  if (all(1 %in% rows) && all(c(1,3) %in% cols)) {
    gt <- gtable_add_grob(gt, g3, 1, 1, 1, length(cols))
  }
  if (all(1:2 %in% rows) && all(c(1,2) %in% cols)) {
    gt <- gtable_add_grob(gt, g4, 1, 1, 2, 2)
  }
  gt
}

test_that("Indexing with grobs that span cells", {

  # Make a gtable with two grobs that span cells
  gt <- make_span_gt(1:2, 1:3)

  # Indexing in ways that don't change gt
  expect_true(equal_gtable(gt, gt[1:2, 1:3]))

  # If a cell at the end of a grob is dropped, drop the grob
  # These should drop all grobs
  expect_true(equal_gtable(gt[1, 2], make_span_gt(1, 2)))
  expect_equal(length(gt[1, 2]$grobs), 0)
  expect_true(equal_gtable(gt[1:2, 2], make_span_gt(1:2, 2)))
  expect_equal(length(gt[1:2, 2]$grobs), 0)

  # These should preserve one of the grobs
  expect_true(equal_gtable(gt[1:2, 1:2], make_span_gt(1:2, 1:2)))
  expect_equal(length(gt[1:2, 1:2]$grobs), 1)
  expect_true(equal_gtable(gt[1, 1:3], make_span_gt(1, 1:3)))
  expect_equal(length(gt[1, 1:3]$grobs), 1)

  # If a cell in the middle of a grob is dropped, don't drop the grob
  expect_true(equal_gtable(gt[1, c(1,3)], make_span_gt(1, c(1,3))))
  expect_equal(length(gt[1, c(1,3)]$grobs), 1)

  # Currently undefined behavior:
  # What happens when you do repeat rows/cols, like gt[1, c(1,1,1,3)] ?
  # What happens when order is non-monotonic, like gt[1, c(3,1,2)] ?
})

gt <- gtable(widths = unit(1, "null"), heights = unit(rep(1, 4), "null"))
gt <- gtable_add_grob(
  gt,
  list(circleGrob(), rectGrob(), polygonGrob(), linesGrob()),
  t = 1:4,
  l = 1,
  name = c('circle', 'rect', 'polygon', 'lines')
)

test_that("filtering works", {
  gt_fil <- gtable_filter(gt, 'circle')
  expect_equal(gt_fil$layout$name, "circle")
  gt_fil <- gtable_filter(gt, 'circle', invert = TRUE)
  expect_equal(gt_fil$layout$name, c("rect", "polygon", "lines"))
  gt_fil <- gtable_filter(gt, '(circle)|(rect)')
  expect_equal(gt_fil$layout$name, c("circle", "rect"))
  gt_fil <- gtable_filter(gt, '(circle)|(rect)', fixed = TRUE, trim = FALSE)
  expect_length(gt_fil, 0)
})

gt_empty <- gtable(
  widths = unit(rep(1, 4), 'null'),
  heights = unit(rep(1, 4), 'null')
)
gt <- gtable_add_grob(gt_empty, rectGrob(), 1, 1)

test_that("trimming works", {
  expect_equal(dim(gtable_trim(gt)), c(1, 1))
  expect_equal(dim(gtable_trim(gt_empty)), c(0, 0))
})

#' Add row/column spacing.
#'
#' Adds \code{width} space between the columns or \code{height} space between
#' the rows.
#'
#' @name gtable_add_space
#' @param x a gtable object
NULL

#' @param width a vector of units of length 1 or ncol - 1
#' @export
#' @rdname gtable_add_space
gtable_add_col_space <- function(x, width) {
  stopifnot(is.gtable(x))
  n <- ncol(x) - 1
  if (n == 0) return(x)
  
  stopifnot(length(width) == 1 || length(width) == n)
  width <- rep(width, length = n)

  for(i in rev(seq_len(n))) {
    x <- gtable_add_cols(x, width[i], pos = i)
  }

  x
}

#' @param height a vector of units of length 1 or nrow - 1
#' @export
#' @rdname gtable_add_space
gtable_add_row_space <- function(x, height) {
  stopifnot(is.gtable(x))
  n <- nrow(x) - 1
  if (n == 0) return(x)
  
  stopifnot(length(height) == 1 || length(height) == n)
  height <- rep(height, length = n)
  
  for(i in rev(seq_len(n))) {
    x <- gtable_add_rows(x, height[i], pos = i)
  }
  
  x
}

library(testthat)

# Find location of a grob
gtable_find <- function(x, grob) {
  pos <- vapply(x$grobs, identical, logical(1), grob)
  x$layout[pos, ]
}

loc_df <- function(t, l, b, r) {
  data.frame(t, l, b, r, z = 1, clip = "on", name = "layout", 
    stringsAsFactors = FALSE)
}

context("gtable")

test_that("Number of rows grows with add_rows", {
  layout <- gtable()
  expect_that(nrow(layout), equals(0))

  layout <- gtable_add_rows(layout, unit(1, "cm"))
  expect_that(nrow(layout), equals(1))

  layout <- gtable_add_rows(layout, unit(1, "cm"))
  layout <- gtable_add_rows(layout, unit(1, "cm"))
  expect_that(nrow(layout), equals(3))

  layout <- gtable_add_rows(layout, unit(1:2, "cm"))
  expect_that(nrow(layout), equals(5))
})


test_that("Number of columns grows with add_cols", {
  layout <- gtable()
  expect_that(ncol(layout), equals(0))

  layout <- gtable_add_cols(layout, unit(1, "cm"))
  expect_that(ncol(layout), equals(1))

  layout <- gtable_add_cols(layout, unit(c(1, 1), "cm"))
  expect_that(ncol(layout), equals(3))

  layout <- gtable_add_cols(layout, unit(1:2, "cm"))
  expect_that(ncol(layout), equals(5))
})


test_that("Setting and getting works", {
  layout <- gtable_add_cols(gtable_add_rows(gtable(), cm), cm)
  
  layout <- gtable_add_grob(layout, grob1, 1, 1)
  loc <- gtable_find(layout, grob1)
  
  expect_that(nrow(loc), equals(1))
  expect_that(loc$t, equals(1))
  expect_that(loc$r, equals(1))
  expect_that(loc$b, equals(1))
  expect_that(loc$l, equals(1))
})

test_that("Spanning grobs continue to span after row insertion", {  
  layout <- gtable_add_cols(gtable_add_rows(gtable(), rep(cm, 3)), rep(cm, 3))
  layout <- gtable_add_grob(layout, grob1, 1, 1, 3, 3)
  
  within <- gtable_add_rows(gtable_add_cols(layout, cm, pos = 2), cm, pos = 2)  
  loc <- gtable_find(within, grob1)
  
  expect_that(loc, equals(loc_df(t = 1, l = 1, b = 4, r = 4)))
  
  top_left <- layout
  top_left <- gtable_add_cols(top_left, cm, pos = 0)
  top_left <- gtable_add_rows(top_left, cm, pos = 0)

  loc <- gtable_find(top_left, grob1)
  expect_that(loc, equals(loc_df(t = 2, l = 2, b = 4, r = 4)))
  
  bottom_right <- layout
  bottom_right <- gtable_add_cols(bottom_right, cm)
  bottom_right <- gtable_add_rows(bottom_right, cm)

  loc <- gtable_find(bottom_right, grob1)
  expect_that(loc, equals(loc_df(t = 1, l = 1, b = 3, r = 3)))
})


test_that("n + 1 new rows/cols after spacing", {
  layout <- gtable()
  layout <- gtable_add_rows(layout, rep(cm, 3))
  layout <- gtable_add_cols(layout, rep(cm, 3))
  
  layout <- gtable_add_col_space(layout, cm)
  expect_that(ncol(layout), equals(5))
  
  layout <- gtable_add_row_space(layout, cm)
  expect_that(ncol(layout), equals(5))
})

test_that("Spacing adds rows/cols in correct place", {
  layout <- gtable()
  layout <- gtable_add_rows(layout, rep(cm, 2))
  layout <- gtable_add_cols(layout, rep(cm, 2))
  
  layout <- gtable_add_col_space(layout, null)
  layout <- gtable_add_row_space(layout, null)
  
  expect_that(as.vector(layout$heights), equals(rep(1, 3)))
  expect_that(attr(layout$heights, "unit"), equals(c("cm", "null", "cm")))

  expect_that(as.vector(layout$widths), equals(rep(1, 3)))
  expect_that(attr(layout$widths, "unit"), equals(c("cm", "null", "cm")))
  
})

test_that("Negative positions place from end", {
  layout <- gtable()
  layout <- gtable_add_rows(layout, rep(cm, 3))
  layout <- gtable_add_cols(layout, rep(cm, 3))
  
  col_span <- gtable_add_grob(layout, grob1, t = 1, l = 1, r = -1)
  expect_that(gtable_find(col_span, grob1), 
    equals(loc_df(t = 1, l = 1, b = 1, r = 3)))

  row_span <- gtable_add_grob(layout, grob1, t = 1, l = 1, b = -1)
  expect_that(gtable_find(row_span, grob1), 
    equals(loc_df(t = 1, l = 1, b = 3, r = 1)))
})

test_that("Adding multiple grobs", {
  grobs <- rep(list(grob1), 8)

  # With z = Inf, and t value for each grob
  tval <- c(1, 2, 3, 1, 2, 3, 1, 2)
  layout <- gtable_add_cols(gtable_add_rows(gtable(), rep(cm, 3)), rep(cm, 3))
  layout <- gtable_add_grob(layout, grobs, tval, 1, 3, 3, z = Inf)
  expect_equal(layout$layout$t, tval)
  expect_equal(layout$layout$z, 1:8)

  # With z = -Inf
  layout <- gtable_add_cols(gtable_add_rows(gtable(), rep(cm, 3)), rep(cm, 3))
  layout <- gtable_add_grob(layout, grobs, 1, 1, 3, 3, z = -Inf)
  expect_equal(layout$layout$z, -7:0)

  # Mixing Inf and non-Inf z values
  zval <- c(Inf, Inf, 6, 0, -Inf, Inf, -2, -Inf)
  layout <- gtable_add_cols(gtable_add_rows(gtable(), rep(cm, 3)), rep(cm, 3))
  layout <- gtable_add_grob(layout, grobs, 1, 1, 3, 3, z = zval)
  expect_equal(layout$layout$z, c(7, 8, 6, 0, -4, 9, -2, -3))

  # Error if inputs are not length 1 or same length as grobs
  layout <- gtable_add_cols(gtable_add_rows(gtable(), rep(cm, 3)), rep(cm, 3))
  expect_error(gtable_add_grob(layout, grobs, c(1:3), 1, 3, 3))
  expect_error(gtable_add_grob(layout, grobs, tval, 1:2, 3, 3))
  expect_error(gtable_add_grob(layout, grobs, tval, 1, 3, 3, z = 1:4))

})

library(testthat)

# Find location of a grob
gtable_find <- function(x, grob) {
  pos <- vapply(x$grobs, identical, logical(1), grob)
  x$layout[pos, ]
}

loc_df <- function(t, l, b, r) {
  data.frame(t, l, b, r,
    z = 1, clip = "on", name = "layout",
    stringsAsFactors = FALSE
  )
}

test_that("Number of rows grows with add_rows", {
  layout <- gtable()
  expect_equal(nrow(layout), 0)

  layout <- gtable_add_rows(layout, unit(1, "cm"))
  expect_equal(nrow(layout), 1)

  layout <- gtable_add_rows(layout, unit(1, "cm"))
  layout <- gtable_add_rows(layout, unit(1, "cm"))
  expect_equal(nrow(layout), 3)

  layout <- gtable_add_rows(layout, unit(1:2, "cm"))
  expect_equal(nrow(layout), 5)
})


test_that("Number of columns grows with add_cols", {
  layout <- gtable()
  expect_equal(ncol(layout), 0)

  layout <- gtable_add_cols(layout, unit(1, "cm"))
  expect_equal(ncol(layout), 1)

  layout <- gtable_add_cols(layout, unit(c(1, 1), "cm"))
  expect_equal(ncol(layout), 3)

  layout <- gtable_add_cols(layout, unit(1:2, "cm"))
  expect_equal(ncol(layout), 5)
})


test_that("Setting and getting works", {
  layout <- gtable_add_cols(gtable_add_rows(gtable(), cm), cm)

  layout <- gtable_add_grob(layout, grob1, 1, 1)
  loc <- gtable_find(layout, grob1)

  expect_equal(nrow(loc), 1)
  expect_equal(loc$t, 1)
  expect_equal(loc$r, 1)
  expect_equal(loc$b, 1)
  expect_equal(loc$l, 1)
})

test_that("Spanning grobs continue to span after row insertion", {
  layout <- gtable_add_cols(gtable_add_rows(gtable(), rep(cm, 3)), rep(cm, 3))
  layout <- gtable_add_grob(layout, grob1, 1, 1, 3, 3)

  within <- gtable_add_rows(gtable_add_cols(layout, cm, pos = 2), cm, pos = 2)
  loc <- gtable_find(within, grob1)

  expect_equal(loc, loc_df(t = 1, l = 1, b = 4, r = 4))

  top_left <- layout
  top_left <- gtable_add_cols(top_left, cm, pos = 0)
  top_left <- gtable_add_rows(top_left, cm, pos = 0)

  loc <- gtable_find(top_left, grob1)
  expect_equal(loc, loc_df(t = 2, l = 2, b = 4, r = 4))

  bottom_right <- layout
  bottom_right <- gtable_add_cols(bottom_right, cm)
  bottom_right <- gtable_add_rows(bottom_right, cm)

  loc <- gtable_find(bottom_right, grob1)
  expect_equal(loc, loc_df(t = 1, l = 1, b = 3, r = 3))
})


test_that("n + 1 new rows/cols after spacing", {
  layout <- gtable()
  layout <- gtable_add_rows(layout, rep(cm, 3))
  layout <- gtable_add_cols(layout, rep(cm, 3))

  layout <- gtable_add_col_space(layout, cm)
  expect_equal(ncol(layout), 5)

  layout <- gtable_add_row_space(layout, cm)
  expect_equal(ncol(layout), 5)
})

test_that("Spacing adds rows/cols in correct place", {
  layout <- gtable()
  layout <- gtable_add_rows(layout, rep(cm, 2))
  layout <- gtable_add_cols(layout, rep(cm, 2))

  layout <- gtable_add_col_space(layout, null)
  layout <- gtable_add_row_space(layout, null)

  expect_equal(as.vector(layout$heights), rep(1, 3))
  expect_equal(sub('1', '', as.character(layout$heights)), c("cm", "null", "cm"))

  expect_equal(as.vector(layout$widths), rep(1, 3))
  expect_equal(sub('1', '', as.character(layout$widths)), c("cm", "null", "cm"))
})

test_that("Negative positions place from end", {
  layout <- gtable()
  layout <- gtable_add_rows(layout, rep(cm, 3))
  layout <- gtable_add_cols(layout, rep(cm, 3))

  col_span <- gtable_add_grob(layout, grob1, t = 1, l = 1, r = -1)
  expect_equal(
    gtable_find(col_span, grob1),
    loc_df(t = 1, l = 1, b = 1, r = 3)
  )

  row_span <- gtable_add_grob(layout, grob1, t = 1, l = 1, b = -1)
  expect_equal(
    gtable_find(row_span, grob1),
    loc_df(t = 1, l = 1, b = 3, r = 1)
  )
})

test_that("Adding multiple grobs", {
  grobs <- rep(list(grob1), 8)

  # With z = Inf, and t value for each grob
  tval <- c(1, 2, 3, 1, 2, 3, 1, 2)
  layout <- gtable_add_cols(gtable_add_rows(gtable(), rep(cm, 3)), rep(cm, 3))
  layout <- gtable_add_grob(layout, grobs, tval, 1, 3, 3, z = Inf)
  expect_equal(layout$layout$t, tval)
  expect_equal(layout$layout$z, 1:8)

  # With z = -Inf
  layout <- gtable_add_cols(gtable_add_rows(gtable(), rep(cm, 3)), rep(cm, 3))
  layout <- gtable_add_grob(layout, grobs, 1, 1, 3, 3, z = -Inf)
  expect_equal(layout$layout$z, -7:0)

  # Mixing Inf and non-Inf z values
  zval <- c(Inf, Inf, 6, 0, -Inf, Inf, -2, -Inf)
  layout <- gtable_add_cols(gtable_add_rows(gtable(), rep(cm, 3)), rep(cm, 3))
  layout <- gtable_add_grob(layout, grobs, 1, 1, 3, 3, z = zval)
  expect_equal(layout$layout$z, c(7, 8, 6, 0, -4, 9, -2, -3))

  # Error if inputs are not length 1 or same length as grobs
  layout <- gtable_add_cols(gtable_add_rows(gtable(), rep(cm, 3)), rep(cm, 3))
  expect_snapshot(gtable_add_grob(layout, grobs, c(1:3), 1, 3, 3), error = TRUE)
  expect_snapshot(gtable_add_grob(layout, grobs, tval, 1:2, 3, 3), error = TRUE)
  expect_snapshot(gtable_add_grob(layout, grobs, tval, 1, 3, 3, z = 1:4), error = TRUE)
})

#' Add new rows in specified position.
#' 
#' @param x a \code{\link{gtable}} object
#' @param heights a unit vector giving the heights of the new rows
#' @param pos new row will be added below this position. Defaults to
#'   adding row on bottom. \code{0} adds on the top.
#' @export
#' @examples
#' rect <- rectGrob(gp = gpar(fill = "#00000080"))
#' tab <- gtable(unit(rep(1, 3), "null"), unit(rep(1, 3), "null"))
#' tab <- gtable_add_grob(tab, rect, t = 1, l = 1, r = 3)
#' tab <- gtable_add_grob(tab, rect, t = 1, b = 3, l = 1)
#' tab <- gtable_add_grob(tab, rect, t = 1, b = 3, l = 3)
#' dim(tab)
#' plot(tab)
#'
#' # Grobs will continue to span over new rows if added in the middle
#' tab2 <- gtable_add_rows(tab, unit(1, "null"), 1)
#' dim(tab2)
#' plot(tab2)
#'
#' # But not when added to top (0) or bottom (-1, the default)
#' tab3 <- gtable_add_rows(tab, unit(1, "null"))
#' tab3 <- gtable_add_rows(tab3, unit(1, "null"), 0)
#' dim(tab3)
#' plot(tab3)
gtable_add_rows <- function(x, heights, pos = -1) {
  stopifnot(is.gtable(x))
  stopifnot(length(pos) == 1)
  n <- length(heights)
  
  pos <- neg_to_pos(pos, nrow(x))
  
  # Shift existing rows down
  x$heights <- insert.unit(x$heights, heights, pos)
  x$layout$t <- ifelse(x$layout$t > pos, x$layout$t + n, x$layout$t)
  x$layout$b <- ifelse(x$layout$b > pos, x$layout$b + n, x$layout$b)

  x
}

#' Add new columns in specified position.
#' 
#' @param x a \code{\link{gtable}} object
#' @param widths a unit vector giving the widths of the new columns
#' @param pos new row will be added below this position. Defaults to
#'   adding col on right. \code{0} adds on the left.
#' @export
#' @examples
#' rect <- rectGrob(gp = gpar(fill = "#00000080"))
#' tab <- gtable(unit(rep(1, 3), "null"), unit(rep(1, 3), "null"))
#' tab <- gtable_add_grob(tab, rect, t = 1, l = 1, r = 3)
#' tab <- gtable_add_grob(tab, rect, t = 1, b = 3, l = 1)
#' tab <- gtable_add_grob(tab, rect, t = 1, b = 3, l = 3)
#' dim(tab)
#' plot(tab)
#'
#' # Grobs will continue to span over new rows if added in the middle
#' tab2 <- gtable_add_cols(tab, unit(1, "null"), 1)
#' dim(tab2)
#' plot(tab2)
#'
#' # But not when added to left (0) or right (-1, the default)
#' tab3 <- gtable_add_cols(tab, unit(1, "null"))
#' tab3 <- gtable_add_cols(tab3, unit(1, "null"), 0)
#' dim(tab3)
#' plot(tab3)
gtable_add_cols <- function(x, widths, pos = -1) {
  stopifnot(is.gtable(x))
  stopifnot(length(pos) == 1)
  n <- length(widths)
  
  pos <- neg_to_pos(pos, ncol(x))
  
  # Shift existing columns right
  x$widths <- insert.unit(x$widths, widths, pos)
  x$layout$l <- ifelse(x$layout$l > pos, x$layout$l + n, x$layout$l)
  x$layout$r <- ifelse(x$layout$r > pos, x$layout$r + n, x$layout$r)
  
  x
}

context("Bind")

test_that("Number of rows grow with rbind", {
  
  lay1 <- gtable_add_rows(gtable(), cm)
  lay2 <- gtable_add_rows(gtable(), rep(cm, 2))
  
  expect_that(nrow(rbind(lay1, lay2)), equals(3))
  expect_that(nrow(rbind(lay2, lay1)), equals(3))
})

test_that("Number of cols grow with cbind", {

  lay1 <- gtable_add_cols(gtable(), cm)
  lay2 <- gtable_add_cols(gtable(), rep(cm, 2))
  
  expect_that(ncol(cbind(lay1, lay2)), equals(3))
  expect_that(ncol(cbind(lay2, lay1)), equals(3))
})

test_that("Heights and widths vary with size parameter", {
  col1 <- gtable_col("col1", list(grob1), cm, cm)
  col2 <- gtable_col("col1", list(grob1), cm2, cm2)

  expect_equal(cbind(col1, col2, size = "first")$heights, cm)
  expect_equal(cbind(col1, col2, size = "last")$heights, cm2)
  expect_equal(cbind(col1, col2, size = "min")$heights, cm)
  expect_equal(cbind(col1, col2, size = "max")$heights, cm2)

  expect_equal(rbind(col1, col2, size = "first")$widths, cm)
  expect_equal(rbind(col1, col2, size = "last")$widths, cm2)
  expect_equal(rbind(col1, col2, size = "min")$widths, cm)
  expect_equal(rbind(col1, col2, size = "max")$widths, cm2)
})

#' Visualise the layout of a gtable.
#'
#' This function is a simple wrapper around [grid::grid.show.layout()] that
#' allows you to inspect the layout of the gtable.
#'
#' @param x a gtable object
#' @inheritDotParams grid::grid.show.layout
#'
#' @export
#'
#' @examples
#' gt <- gtable(widths = grid::unit(c(1, 0.5, 2), c("null", "cm", "null")),
#'              heights = grid::unit(c(0.2, 1, 3), c("inch", "null", "cm")))
#' gtable_show_layout(gt)
#'
gtable_show_layout <- function(x, ...) {
  if (!is.gtable(x)) stop("x must be a gtable", call. = FALSE)

  grid.show.layout(gtable_layout(x), ...)
}

gtable_layout <- function(x) {
  if (!is.gtable(x)) stop("x must be a gtable", call. = FALSE)

  grid.layout(
    nrow = length(x$heights), heights = x$heights,
    ncol = length(x$widths), widths = x$widths,
    respect = x$respect
  )
}

vpname <- function(row) {
  row <- unclass(row)
  paste(row$name, ".", row$t, "-", row$r, "-", row$b, "-", row$l, sep = "")
}

#' @export
widthDetails.gtable <- function(x) absolute.size(gtable_width(x))

#' @export
heightDetails.gtable <- function(x) absolute.size(gtable_height(x))

#' @export
makeContext.gtable <- function(x) {
  layoutvp <- viewport(layout = gtable_layout(x), name = x$name)
  if (is.null(x$vp)) {
    x$vp <- layoutvp
  } else {
    x$vp <- vpStack(x$vp, layoutvp)
  }
  x
}

#' @export
makeContent.gtable <- function(x) {
  children_vps <- mapply(child_vp,
    vp_name = vpname(x$layout),
    t = .subset2(x$layout, "t"), r = .subset2(x$layout, "r"),
    b = .subset2(x$layout, "b"), l = .subset2(x$layout, "l"),
    clip = x$layout$clip,
    SIMPLIFY = FALSE
  )
  x$grobs <- mapply(wrap_gtableChild, x$grobs, children_vps,
    SIMPLIFY = FALSE
  )
  setChildren(x, do.call("gList", x$grobs[order(.subset2(x$layout, "z"))]))
}

#' @export
makeContext.gTableChild <- function(x) {
  if (is.null(x$vp)) {
    x$vp <- x$wrapvp
  } else {
    x$vp <- vpStack(x$wrapvp, x$vp)
  }
  # A gTableChild extends an arbitrary grob class
  # so allow existing makeContext() behaviour of
  # original grob class to still occur
  NextMethod()
}

# Return the viewport for a child grob in a gtable
child_vp <- function(vp_name, t, r, b, l, clip) {
  viewport(
    name = vp_name, layout.pos.row = t:b,
    layout.pos.col = l:r, clip = clip
  )
}

# Turn a grob into a gtableChild, and store information about the
# viewport used within the gtable
wrap_gtableChild <- function(grob, vp) {
  grob$wrapvp <- vp
  grob$name <- vp$name
  class(grob) <- c("gTableChild", class(grob))
  grob
}

# Code does not currently work - need to thinking about how indexing a gtable
# should work in more detail.  How do the grobs move around?

#  Join two gtables together based on row/column names.
#
#  @inheritParams gtable_align
#  @param along dimension to align along, \code{1} = rows, \code{2} = cols.
#    Join will occur perpendicular to this direction.
#  @examples
#  rect <- rectGrob(gp = gpar(fill = "black"))
#  circ <- circleGrob(gp = gpar(fill = "red"))
#  a <- gtable_col("a", list(rect, circ), width = unit(5, "cm"))
#  rownames(a) <- c("top", "mid")
#  b <- gtable_col("b", list(circ, rect), width = unit(5, "cm"))
#  rownames(b) <- c("mid", "bot")
#
#  # Commented out example below because it causes R CMD check to fail
#  # when this function is not exported. Uncomment when this function
#  # is fixed and exported again.
#  # gtable_join(a, b)
gtable_join <- function(x, y, along = 1L, join = "left") {
  aligned <- gtable_align(x, y, along = along, join = join)
  switch(along,
    cbind(aligned$x, aligned$y), 
    rbind(aligned$x, aligned$y),
    stop("along > 2 no implemented"))
}

#  Align two gtables based on their row/col names.
#
#  @param x \code{\link{gtable}}
#  @param y \code{\link{gtable}}
#  @param along dimension to align along, \code{1} = rows, \code{2} = cols.
#  @param join when x and y have different names, how should the difference
#    be resolved? \code{inner} keep names that appear in both, \code{outer}
#    keep names that appear in either, \code{left} keep names from \code{x},
#    and \code{right} keep names from \code{y}.
#  @seealso \code{\link{gtable_join}} to return the two gtables combined
#    in to a single gtable.
#  @return a list with elements \code{x} and \code{y} corresponding to the
#    input gtables with extra rows/columns so that they now align.
gtable_align <- function(x, y, along = 1L, join = "left") {
  join <- match.arg(join, c("left", "right", "inner", "outer"))
  
  names_x <- dimnames(x)[[along]]
  names_y <- dimnames(y)[[along]]
  
  if (is.null(names_x) || is.null(names_y)) {
    stop("Both gtables must have names along dimension to be aligned")
  }
  
  idx <- switch(join,
    left = names_x,
    right = names_y, 
    inner = intersect(names_x, names_y),
    outer = union(names_x, names_y)
  )
  
  list(
    x = gtable_reindex(x, idx, along), 
    y = gtable_reindex(y, idx, along)
  )
}

#  Reindex a gtable.
#
#  @keywords internal
#  @examples
#  gt <- gtable(heights = unit(rep(1, 3), "cm"), rownames = c("a", "b", "c"))
#  rownames(gtable:::gtable_reindex(gt, c("a", "b", "c")))
#  rownames(gtable:::gtable_reindex(gt, c("a", "b")))
#  rownames(gtable:::gtable_reindex(gt, c("a")))
#  rownames(gtable:::gtable_reindex(gt, c("a", "d", "e")))
gtable_reindex <- function(x, index, along = 1) {
  stopifnot(is.character(index))
  if (length(dim(x)) > 2L || along > 2L) {
    stop("reindex only supports 2d objects")
  }
  old_index <- rownames(x)
  stopifnot(!is.null(old_index))
  
  if (identical(index, old_index)) {
    return(x)
  }
  
  if (!(old_index %contains% index)) {
    missing <- setdiff(index, old_index)
    # Create and add dummy space rows

    if (along == 1L) {
      spacer <- gtable(
        widths = unit(rep(0, ncol(x)), "cm"), 
        heights = rep_along(unit(0, "cm"), missing),
        rownames = missing)
      # spacer <- x[rep_along(NA, missing), ]
      # rownames(spacer) <- missing
      x <- rbind(x, spacer, size = "first")
    } else {
      spacer <- gtable(
        heights = unit(rep(0, ncol(x)), "cm"), 
        widths = rep_along(unit(0, "cm"), missing),
        colnames = missing)
      # spacer <- x[, rep_along(NA, missing)]
      # colnames(spacer) <- missing
      x <- cbind(x, spacer, size = "first")
    }
  }
  
  # Reorder & subset
  switch(along,
    x[index, ],
    x[, index])
}

"%contains%" <- function(x, y) all(y %in% x)

rep_along <- function(x, y) {
  if (length(y) == 0) return(NULL)
  rep(x, length(y))
}

#' Create a single column gtable.
#'
#' @inheritParams gtable
#' @inheritParams gtable_add_grob
#' @param width a unit vector giving the width of this column
#' @param vp a grid viewport object (or NULL).
#' @export
#' @examples
#' a <- rectGrob(gp = gpar(fill = "red"))
#' b <- circleGrob()
#' c <- linesGrob()
#' gt <- gtable_col("demo", list(a, b, c))
#' gt
#' plot(gt)
#' gtable_show_layout(gt)
gtable_col <- function(name, grobs, width = NULL, heights = NULL,
  z = NULL, vp = NULL) {
  width <- width %||% unit(max(unlist(lapply(grobs, width_cm))), "cm")
  heights <- heights %||% rep(unit(1, "null"), length(grobs))

  # z is either NULL, or a vector of the same length as grobs
  stopifnot(is.null(z) || length(z) == length(grobs))
  if (is.null(z))
    z <- Inf

  table <- gtable(name = name, vp = vp)
  
  table <- gtable_add_rows(table, heights)
  table <- gtable_add_cols(table, width)
  table <- gtable_add_grob(table, grobs, t = seq_along(grobs), l = 1, 
    z = z, clip = "off")
  
  table
}

#' Create a single row gtable.
#'
#' @inheritParams gtable
#' @inheritParams gtable_add_grob
#' @param height a unit vector giving the height of this row
#' @param vp a grid viewport object (or NULL).
#' @export
#' @examples
#' a <- rectGrob(gp = gpar(fill = "red"))
#' b <- circleGrob()
#' c <- linesGrob()
#' gt <- gtable_row("demo", list(a, b, c))
#' gt
#' plot(gt)
#' gtable_show_layout(gt)
gtable_row <- function(name, grobs, height = NULL, widths = NULL,
  z = NULL, vp = NULL) {
  height <- height %||% unit(max(unlist(lapply(grobs, height_cm))), "cm")
  widths <- widths %||% rep(unit(1, "null"), length(grobs))

  # z is either NULL, or a vector of the same length as grobs
  stopifnot(is.null(z) || length(z) == length(grobs))
  if (is.null(z))
    z <- Inf
    
  table <- gtable(name = name, vp = vp)

  table <- gtable_add_cols(table, widths)
  table <- gtable_add_rows(table, height)
  table <- gtable_add_grob(table, grobs, l = seq_along(grobs), t = 1,
    z = z, clip = "off")
  
  table
}

#' Create a gtable from a matrix of grobs.
#'
#' @export
#' @inheritParams gtable
#' @inheritParams gtable_add_grob
#' @param z a numeric matrix of the same dimensions as \code{grobs},
#'   specifying the order that the grobs are drawn.
#' @param vp a grid viewport object (or NULL).
#' @examples
#' a <- rectGrob(gp = gpar(fill = "red"))
#' b <- circleGrob()
#' c <- linesGrob()
#' 
#' row <- matrix(list(a, b, c), nrow = 1)
#' col <- matrix(list(a, b, c), ncol = 1)
#' mat <- matrix(list(a, b, c, nullGrob()), nrow = 2)
#'
#' gtable_matrix("demo", row, unit(c(1, 1, 1), "null"), unit(1, "null"))
#' gtable_matrix("demo", col, unit(1, "null"), unit(c(1, 1, 1), "null"))
#' gtable_matrix("demo", mat, unit(c(1, 1), "null"), unit(c(1, 1), "null"))
#'
#' # Can specify z ordering
#' z <- matrix(c(3, 1, 2, 4), nrow = 2)
#' gtable_matrix("demo", mat, unit(c(1, 1), "null"), unit(c(1, 1), "null"), z = z)
gtable_matrix <- function(name, grobs, widths = NULL, heights = NULL,
  z = NULL, respect = FALSE, clip = "on", vp = NULL) {

  table <- gtable(name = name, respect = respect, vp = vp)

  stopifnot(length(widths) == ncol(grobs))
  stopifnot(length(heights) == nrow(grobs))
  # z is either NULL or a matrix of the same dimensions as grobs
  stopifnot(is.null(z) || identical(dim(grobs), dim(z)))
  if (is.null(z))
    z <- Inf

  table <- gtable_add_cols(table, widths)
  table <- gtable_add_rows(table, heights)
  
  table <- gtable_add_grob(table, grobs, t = c(row(grobs)), l = c(col(grobs)),
    z = as.vector(z), clip = clip)

  table
}

#' Create a row/col spacer gtable.
#'
#' @name gtable_spacer
NULL

#' @param widths unit vector of widths
#' @rdname gtable_spacer
#' @export
gtable_row_spacer <- function(widths) {
  gtable_add_cols(gtable(), widths)
}

#' @param heights unit vector of heights
#' @rdname gtable_spacer
#' @export
gtable_col_spacer <- function(heights) {
  gtable_add_rows(gtable(), heights)
}

context("z-order")

# z tests for gtable_add_grob are in test-layout.r, mixed with other tests


test_that("z order for row, column, and matrix layouts", {
  zorder <- c(3, 1, 2, 4)

  # ==== column ====
  gt <- gtable_col("test", list(grob1, grob2, grob3, grob4))
  # z for positions 1 2 3 4 (left to right) should equal 1:4
  expect_equal(gt$layout$z[gt$layout$t], 1:4)

  gt <- gtable_col("test", list(grob1, grob2, grob3, grob4), z = zorder)
  # z for position 1 2 3 4 (left to right) should equal zorder
  expect_equal(gt$layout$z[gt$layout$t], zorder)

  # ==== row ====
  gt <- gtable_row("test", list(grob1, grob2, grob3, grob4))
  # z for positions 1 2 3 4 (top to bottom) should equal 1:4
  expect_equal(gt$layout$z[gt$layout$l], 1:4)

  gt <- gtable_row("test", list(grob1, grob2, grob3, grob4), z = zorder)
  # z for position 1 2 3 4 (top to bottom) should equal zorder
  expect_equal(gt$layout$z[gt$layout$l], zorder)

  # ==== matrix ====
  gt <- gtable_matrix("test", matrix(list(grob1, grob2, grob3, grob4), 
    nrow = 2), unit(c(1, 1), "null"), unit(c(1, 1), "null"))
  # Get the position. Should be: 1 3
  #                              2 4
  loc <- 2 * (gt$layout$l - 1) + gt$layout$t
  # z for positions 1:4 should equal 1:4
  expect_equal(gt$layout$z[loc], 1:4)

  gt <- gtable_matrix("test", matrix(list(grob1, grob2, grob3, grob4), 
    nrow = 2), unit(c(1, 1), "null"), unit(c(1, 1), "null"),
    z = matrix(zorder, nrow = 2))
  # Get the position. Should be: 1 3
  #                              2 4
  loc <- 2 * (gt$layout$l - 1) + gt$layout$t
  # z for positions 1:4 should equal zorder
  expect_equal(gt$layout$z[loc], zorder)

})


test_that("z_normalise works properly", {
  # Non-integer starting zorder, in funny order
  zorder <- c(0.001, -4, 0, 1e6)
  gt <- gtable_col("test", list(grob1, grob2, grob3, grob4), z = zorder)
  expect_equal(gt$layout$z, zorder)
  gt1 <- z_normalise(gt)
  expect_equal(sort(gt1$layout$z), 1:4)

  # OK with empty layout (zero rows in data frame)
  gt <- gtable(unit(1:3, c("cm")), unit(c(2,4), "cm"))
  gt1 <- z_normalise(gt)
  expect_equal(nrow(gt1$layout), 0)
})



test_that("z_arrange_gtables properly sets z values", {
  gt <- list(
    gtable_col("test1", list(grob1, grob2, grob3), z = c(.9, .3, .6)),
    gtable_col("test2", list(grob4, grob1, grob2), z = c(1, 3, 2)),
    gtable_col("test3", list(grob3, grob4, grob1), z = c(2, 3, 1))
  )

  # Arrange the z values of each gtable
  gt1 <- z_arrange_gtables(gt, c(3, 2, 1))
  expect_equal(gt1[[1]]$layout$z, c(9, 7, 8))
  expect_equal(gt1[[2]]$layout$z, c(4, 6, 5))
  expect_equal(gt1[[3]]$layout$z, c(2, 3, 1))

  # Check that it works with cbind and rbind (which call z_arrange_gtables)
  gt1 <- cbind(gt[[1]], gt[[2]], gt[[3]], z = c(3, 2, 1))
  expect_equal(gt1$layout$z, c(9, 7, 8, 4, 6, 5, 2, 3, 1))
  gt1 <- rbind(gt[[1]], gt[[2]], gt[[3]], z = c(3, 2, 1))
  expect_equal(gt1$layout$z, c(9, 7, 8, 4, 6, 5, 2, 3, 1))
})

#' Add a single grob, possibly spanning multiple rows or columns.
#'
#' This only adds grobs into the table - it doesn't affect the table layout in
#' any way.  In the gtable model, grobs always fill up the complete table
#' cell.  If you want custom justification you might need to define the grob
#' dimension in absolute units, or put it into another gtable that can then be
#' added to the gtable instead of the grob.
#'
#' @param x a [gtable()] object
#' @param grobs a single grob or a list of grobs
#' @param t a numeric vector giving the top extent of the grobs
#' @param l a numeric vector giving the left extent of the grobs
#' @param b a numeric vector giving the bottom extent of the grobs
#' @param r a numeric vector giving the right extent of the grobs
#' @param z a numeric vector giving the order in which the grobs should be
#'   plotted.  Use `Inf` (the default) to plot above or `-Inf`
#'   below all existing grobs. By default positions are on the integers,
#'  giving plenty of room to insert new grobs between existing grobs.
#' @param clip should drawing be clipped to the specified cells
#'   (`"on"`), the entire table (`"inherit"`), or not at all
#'   (`"off"`)
#' @param name name of the grob - used to modify the grob name before it's
#'   plotted.
#'
#' @return A gtable object with the new grob(s) added
#'
#' @family gtable manipulation
#'
#' @export
#'
#' @examples
#' library(grid)
#'
#' gt <- gtable(widths = unit(c(1, 1), 'null'), heights = unit(c(1, 1), 'null'))
#' pts <- pointsGrob(x = runif(5), y = runif(5))
#'
#' # Add a grob to a single cell (top-right cell)
#' gt <- gtable_add_grob(gt, pts, t = 1, l = 2)
#'
#' # Add a grob spanning multiple cells
#' gt <- gtable_add_grob(gt, pts, t = 1, l = 1, b = 2)
#'
#' plot(gt)
#'
gtable_add_grob <- function(x, grobs, t, l, b = t, r = l, z = Inf, clip = "on", name = x$name) {
  check_gtable(x)
  if (is.grob(grobs)) {
    grobs <- list(grobs)
  }
  if (!is_list(grobs) || any(!vapply(grobs, is.grob, logical(1)))) {
    stop_input_type(grobs, "a single grob or a list of grobs")
  }
  n_grobs <- length(grobs)

  if (is.logical(clip)) {
    clip <- ifelse(clip, "on", "off")
  }

  layout <- unclass(x$layout)

  # Check that inputs have the right length
  if (!all(vapply(
    list(t, r, b, l, z, clip, name), len_same_or_1,
    logical(1), n_grobs
  ))) {
    cli::cli_abort("Not all inputs have either length 1 or same length same as {.arg grobs}")
  }

  # If z is just one value, replicate to same length as grobs
  z <- rep(z, length.out = n_grobs)

  # Get the existing z values from x$layout, and new non-Inf z-values
  zval <- c(layout$z, z[!is.infinite(z)])
  if (length(zval) == 0) {
    # If there are no existing finite z values, set these so that
    # -Inf values get assigned ..., -2, -1, 0 and
    # +Inf values get assigned 1, 2, 3, ...
    zmin <- 1
    zmax <- 0
  } else {
    zmin <- min(zval)
    zmax <- max(zval)
  }
  z[z == -Inf] <- zmin - rev(seq_len(sum(z == -Inf)))
  z[z == Inf] <- zmax + seq_len(sum(z == Inf))

  x_row <- length(x$heights)
  x_col <- length(x$widths)

  t <- rep(neg_to_pos(t, x_row), length.out = n_grobs)
  b <- rep(neg_to_pos(b, x_row), length.out = n_grobs)
  l <- rep(neg_to_pos(l, x_col), length.out = n_grobs)
  r <- rep(neg_to_pos(r, x_col), length.out = n_grobs)
  clip <- rep(clip, length.out = n_grobs)
  name <- rep(name, length.out = n_grobs)

  x$grobs <- c(x$grobs, grobs)

  x$layout <- new_data_frame(list(
    t = c(layout$t, t),
    l = c(layout$l, l),
    b = c(layout$b, b),
    r = c(layout$r, r),
    z = c(layout$z, z),
    clip = c(layout$clip, clip),
    name = c(layout$name, name)
  ))

  x
}

#' Add padding around edges of table.
#'
#' @param x a \code{\link{gtable}} object
#' @param padding vector of length 4: top, right, bottom, left.  Normal
#'  recycling rules apply.
#' @export
#' @examples
#' gt <- gtable(unit(1, "null"), unit(1, "null"))
#' gt <- gtable_add_grob(gt, rectGrob(gp = gpar(fill = "black")), 1, 1)
#'
#' plot(gt)
#' plot(cbind(gt, gt))
#' plot(rbind(gt, gt))
#' 
#' pad <- gtable_add_padding(gt, unit(1, "cm"))
#' plot(pad)
#' plot(cbind(pad, pad))
#' plot(rbind(pad, pad))
gtable_add_padding <- function(x, padding) {
  padding <- rep(padding, length = 4)
  
  x <- gtable_add_rows(x, pos = 0, heights = padding[1])
  x <- gtable_add_cols(x, pos = -1, widths = padding[2])
  x <- gtable_add_rows(x, pos = -1, heights = padding[3])
  x <- gtable_add_cols(x, pos = 0, widths = padding[4])
  x
}

#' Visualise the layout of a gtable.
#'
#' This function is a simple wrapper around [grid::grid.show.layout()] that
#' allows you to inspect the layout of the gtable.
#'
#' @param x a gtable object
#' @inheritDotParams grid::grid.show.layout
#'
#' @export
#'
#' @examples
#' gt <- gtable(widths = grid::unit(c(1, 0.5, 2), c("null", "cm", "null")),
#'              heights = grid::unit(c(0.2, 1, 3), c("inch", "null", "cm")))
#' gtable_show_layout(gt)
#'
gtable_show_layout <- function(x, ...) {
  grid.show.layout(gtable_layout(x), ...)
}

gtable_layout <- function(x) {
  check_gtable(x)

  grid.layout(
    nrow = length(x$heights), heights = x$heights,
    ncol = length(x$widths), widths = x$widths,
    respect = x$respect
  )
}

vpname <- function(row) {
  row <- unclass(row)
  paste0(row$name, ".", row$t, "-", row$r, "-", row$b, "-", row$l)
}

#' @export
widthDetails.gtable <- function(x) absolute.size(gtable_width(x))

#' @export
heightDetails.gtable <- function(x) absolute.size(gtable_height(x))

#' @export
makeContext.gtable <- function(x) {
  layoutvp <- viewport(layout = gtable_layout(x), name = x$name)
  if (is.null(x$vp)) {
    x$vp <- layoutvp
  } else {
    x$vp <- vpStack(x$vp, layoutvp)
  }
  x
}

#' @export
makeContent.gtable <- function(x) {
  child_name <- vpname(x$layout)
  children_vps <- mapply(child_vp,
    vp_name = child_name,
    t = .subset2(x$layout, "t"), r = .subset2(x$layout, "r"),
    b = .subset2(x$layout, "b"), l = .subset2(x$layout, "l"),
    clip = .subset2(x$layout, "clip"),
    SIMPLIFY = FALSE
  )
  x$grobs <- mapply(grobTree, x$grobs, name = child_name, vp = children_vps,
    SIMPLIFY = FALSE
  )
  setChildren(x, inject(gList(!!!x$grobs[order(.subset2(x$layout, "z"))])))
}

# Return the viewport for a child grob in a gtable
child_vp <- function(vp_name, t, r, b, l, clip) {
  viewport(
    name = vp_name, layout.pos.row = t:b,
    layout.pos.col = l:r, clip = clip
  )
}

neg_to_pos <- function(x, max) {
  ifelse(x >= 0, x, max + 1 + x)
}

compare.unit <- function(x, y, comp) {
  if (length(x) == 0) return(y)
  if (length(y) == 0) return(x)
  
  x_val <- unclass(x)
  y_val <- unclass(y)
  
  x_unit <- attr(x, "unit")
  y_unit <- attr(x, "unit")
  
  if (!all(x_unit == y_unit)) {
    stop("Comparison of units with different types currently not supported")
  }
  
  unit(comp(x_val, y_val), x_unit)
}


insert.unit <- function (x, values, after = length(x)) {
  lengx <- length(x)
  if (lengx == 0) return(values)
  if (length(values) == 0) return(x)

  if (after <= 0) {
    unit.c(values, x)
  } else if (after >= lengx) {
    unit.c(x, values)
  } else {
    unit.c(x[1L:after], values, x[(after + 1L):lengx])
  }
}

"%||%" <- function(a, b) {
  if (!is.null(a)) a else b
}

width_cm <- function(x) {
  if (is.grob(x)) {
    convertWidth(grobWidth(x), "cm", TRUE)
  } else if (is.list(x)) {
    vapply(x, width_cm, numeric(1))
  } else if (is.unit(x)) {
    convertWidth(x, "cm", TRUE)
  } else {
    stop("Unknown input")
  }  
}
height_cm <- function(x) {
  if (is.grob(x)) {
    convertWidth(grobHeight(x), "cm", TRUE)
  } else if (is.list(x)) {
    vapply(x, height_cm, numeric(1))
  } else if (is.unit(x)) {
    convertHeight(x, "cm", TRUE)
  } else {
    stop("Unknown input")
  }  
}

# Check that x is same length as g, or length 1
len_same_or_1 <- function(x, g) {
  if(length(x) == 1 || length(x) == length(g)) {
    TRUE
  } else {
    FALSE
  }
}

#' Add row/column spacing.
#'
#' Adds `width` space between the columns or `height` space between
#' the rows, effictvely pushing the existing cells apart.
#'
#' @name gtable_add_space
#' @param x a gtable object
#'
#' @return A gtable with the additional rows or columns added
#'
#' @family gtable manipulation
#'
#' @examples
#' library(grid)
#'
#' rect <- rectGrob()
#' rect_mat <- matrix(rep(list(rect), 9), nrow = 3)
#'
#' gt <- gtable_matrix("rects", rect_mat, widths = unit(rep(1, 3), "null"),
#'                     heights = unit(rep(1, 3), "null"))
#'
#' plot(gt)
#'
#' # Add spacing between the grobs
#' # same height between all rows
#' gt <- gtable_add_row_space(gt, unit(0.5, "cm"))
#'
#' # Different width between the columns
#' gt <- gtable_add_col_space(gt, unit(c(0.5, 1), "cm"))
#'
#' plot(gt)
NULL

#' @param width a vector of units of length 1 or ncol - 1
#' @export
#' @rdname gtable_add_space
gtable_add_col_space <- function(x, width) {
  check_gtable(x)
  n <- length(x$widths) - 1
  if (n == 0) return(x)

  if (!(length(width) == 1 || length(width) == n)) {
    cli::cli_abort("{.arg width} must be of length 1 or ncol - 1")
  }
  width <- rep(width, length.out = n)

  for (i in rev(seq_len(n))) {
    x <- gtable_add_cols(x, width[i], pos = i)
  }

  x
}

#' @param height a vector of units of length 1 or nrow - 1
#' @export
#' @rdname gtable_add_space
gtable_add_row_space <- function(x, height) {
  check_gtable(x)
  n <- length(x$heights) - 1
  if (n == 0) return(x)

  if (!(length(height) == 1 || length(height) == n)) {
    cli::cli_abort("{.arg height} must be of length 1 or nrow - 1")
  }
  height <- rep(height, length.out = n)

  for (i in rev(seq_len(n))) {
    x <- gtable_add_rows(x, height[i], pos = i)
  }

  x
}

#' Trim off empty cells.
#'
#' @param x a gtable object
#' @export
#' @examples
#' rect <- rectGrob(gp = gpar(fill = "black"))
#' base <- gtable(unit(c(2, 2, 2), "cm"), unit(c(2, 2, 2), "cm"))
#'
#' center <- gtable_add_grob(base, rect, 2, 2)
#' plot(center)
#' plot(gtable_trim(center))
#'
#' col <- gtable_add_grob(base, rect, 1, 2, 3, 2)
#' plot(col)
#' plot(gtable_trim(col))
#'
#' row <- gtable_add_grob(base, rect, 2, 1, 2, 3)
#' plot(row)
#' plot(gtable_trim(row))
gtable_trim <- function(x) {
  stopifnot(is.gtable(x))
  
  w <- range(x$layout$l, x$layout$r)
  h <- range(x$layout$t, x$layout$b)
  
  x$widths <- x$widths[seq.int(w[1], w[2])]
  x$heights <- x$heights[seq.int(h[1], h[2])]
  
  x$layout$l <- x$layout$l - w[1] + 1
  x$layout$r <- x$layout$r - w[1] + 1
  x$layout$t <- x$layout$t - h[1] + 1
  x$layout$b <- x$layout$b - h[1] + 1
  
  x
}

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## -----------------------------------------------------------------------------
#  library(ggplot2)
#  library(profvis)
#  
#  p <- ggplot(mtcars, aes(mpg, disp)) +
#    geom_point() +
#    facet_grid(gear~cyl)
#  
#  p_build <- ggplot_build(p)
#  
#  profile <- profvis(for (i in seq_len(100)) ggplot_gtable(p_build))
#  
#  profile

## ---- eval=FALSE, include=FALSE-----------------------------------------------
#  saveRDS(profile, file.path('profilings', paste0(packageVersion('gtable'), '.rds')))

# Fast data.frame constructor
# No checking, recycling etc. unless asked for
new_data_frame <- function(x, n = NULL) {
  if (is.null(n)) {
    n <- if (length(x) == 0) 0 else length(x[[1]])
  }

  class(x) <- "data.frame"

  attr(x, "row.names") <- .set_row_names(n)
  x
}

validate_data_frame <- function(x) {
  if (length(unique(lengths(x))) != 1) {
    cli::cli_abort('All elements in a data.frame must be of equal length')
  }
  if (is.null(names(x))) {
    cli::cli_abort('columns must be named')
  }
}

test_that("new_data_frame returns a data.frame of expected dimensions", {
  df <- new_data_frame(list(x = 1:3, y = c("a", "b", "c")))

  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 3)
  expect_equal(ncol(df), 2)
  expect_named(df, c("x", "y"))
})

#' gtable
#'
#' @import grid
#' @docType package
#' @name gtable
NULL

#' Create a new grob table.
#'
#' A grob table captures all the information needed to layout grobs in a table
#' structure. It supports row and column spanning, offers some tools to
#' automatically figure out the correct dimensions, and makes it easy to
#' align and combine multiple tables.
#'
#' Each grob is put in its own viewport - grobs in the same location are 
#' not combined into one cell. Each grob takes up the entire cell viewport
#' so justification control is not available.
#'
#' It constructs both the viewports and the gTree needed to display the table.
#'
#' @section Components:
#'
#' There are three basics components to a grob table: the specification of 
#' table (cell heights and widths), the layout (for each grob, its position,
#' name and other settings), and global parameters.
#' 
#' It's easier to understand how \code{gtable} works if in your head you keep
#' the table separate from it's contents.  Each cell can have 0, 1, or many
#' grobs inside. Each grob must belong to at least one cell, but can span
#' accross many cells.
#'
#' @section Layout:
#' 
#' The layout details are stored in a data frame with one row for each grob,
#' and columns:
#'
#' \itemize{
#'   \item \code{t} top extent of grob
#'   \item \code{r} right extent of grob
#'   \item \code{b} bottom extent of
#'   \item \code{l} left extent of grob
#'   \item \code{z} the z-order of the grob - used to reorder the grobs 
#'     before they are rendered
#'   \item \code{clip} a string, specifying how the grob should be clipped:
#'     either \code{"on"}, \code{"off"} or \code{"inherit"}
#'   \item \code{name}, a character vector used to name each grob and its
#'     viewport
#' }
#' 
#' You should not need to modify this data frame directly - instead use
#' functions like \code{gtable_add_grob}.
#'
#' @param widths a unit vector giving the width of each column
#' @param heights a unit vector giving the height of each row
#' @param respect a logical vector of length 1: should the aspect ratio of 
#'   height and width specified in null units be respected.  See
#'   \code{\link{grid.layout}} for more details
#' @param name a string giving the name of the table. This is used to name
#'   the layout viewport
#' @param rownames,colnames character vectors of row and column names, used
#'   for characteric subsetting, particularly for \code{gtable_align},
#'   and \code{gtable_join}.
#' @param vp a grid viewport object (or NULL).
#' @export
#' @aliases gtable-package
#' @seealso \code{\link{gtable_row}}, \code{\link{gtable_col}} and
#'   \code{\link{gtable_matrix}} for convenient ways of creating gtables.
#' @examples
#' a <- gtable(unit(1:3, c("cm")), unit(5, "cm"))
#' a
#' gtable_show_layout(a)
#'
#' # Add a grob:
#' rect <- rectGrob(gp = gpar(fill = "black"))
#' a <- gtable_add_grob(a, rect, 1, 1)
#' a
#' plot(a)
#' 
#' # gtables behave like matrices:
#' dim(a)
#' t(a)
#' plot(t(a))
#' 
#' # when subsetting, grobs are retained if their extents lie in the 
#' # rows/columns that retained.
#' 
#' b <- gtable(unit(c(2, 2, 2), "cm"), unit(c(2, 2, 2), "cm"))
#' b <- gtable_add_grob(b, rect, 2, 2)
#' b[1, ]
#' b[, 1]
#' b[2, 2]
#'
#' # gtable have row and column names
#' rownames(b) <- 1:3
#' rownames(b)[2] <- 200
#' colnames(b) <- letters[1:3]
#' dimnames(b)
gtable <- function(widths = list(), heights = list(), respect = FALSE,
  name = "layout", rownames = NULL, colnames = NULL, vp = NULL) {
  
  if (length(widths) > 0) {
    stopifnot(is.unit(widths))
    stopifnot(is.null(colnames) || length(colnames == length(widths)))
  }
  if (length(heights) > 0) {
    stopifnot(is.unit(heights))
    stopifnot(is.null(rownames) || length(rownames == length(heights)))
  }
  
  layout <- data.frame(
    t = numeric(), l = numeric(), b = numeric(), r = numeric(), z = numeric(),
    clip = character(), name = character(), stringsAsFactors = FALSE)

  if (!is.null(vp)) {
    vp <- viewport(name = name,
      x = vp$x, y = vp$y,
      width = vp$width, height = vp$height,
      just = vp$just, gp = vp$gp, xscale = vp$xscale,
      yscale = vp$yscale, angle = vp$angle, clip = vp$clip)
  }

  grob(
    grobs = list(), layout = layout, widths = widths,
    heights = heights, respect = respect, name = name,
    rownames = rownames, colnames = colnames, vp = vp,
    cl = "gtable")
}

#' Print a gtable object
#'
#' @param x A gtable object.
#' @param zsort Sort by z values? Default \code{FALSE}.
#' @param ... Other arguments (not used by this method).
#' @export
#' @method print gtable
print.gtable <- function(x, zsort = FALSE, ...) {
  cat("TableGrob (", nrow(x), " x ", ncol(x), ") \"", x$name, "\": ", 
    length(x$grobs), " grobs\n", sep = "")
  
  if (nrow(x$layout) == 0) return()
  
  pos <- as.data.frame(format(as.matrix(x$layout[c("t", "r", "b", "l")])), 
    stringsAsFactors = FALSE)
  grobNames <- vapply(x$grobs, as.character, character(1))

  info <- data.frame(
    z = x$layout$z,
    cells = paste("(", pos$t, "-", pos$b, ",", pos$l, "-", pos$r, ")", sep =""),
    name = x$layout$name,
    grob = grobNames
    )
  if (zsort) info <- info[order(x$layout$z), ]

  print(info)
}


#' @S3method dim gtable
dim.gtable <- function(x) c(length(x$heights), length(x$widths))

#' @S3method dimnames gtable
dimnames.gtable <- function(x, ...) list(x$rownames, x$colnames)

#' @S3method dimnames<- gtable
"dimnames<-.gtable" <- function(x, value) {
  x$rownames <- value[[1]]
  x$colnames <- value[[2]]
  
  if (anyDuplicated(x$rownames)) stop("rownames must be distinct", 
    call. = FALSE)
  if (anyDuplicated(x$colnames)) stop("colnames must be distinct", 
    call. = FALSE)
  
  x
}

#' @S3method plot gtable
plot.gtable <- function(x, ...) {
  grid.newpage()
  grid.rect(gp = gpar(fill = "grey95"))
  grid <- seq(0, 1, length = 20)
  grid.grill(h = grid, v = grid, gp = gpar(col = "white"))
  grid.draw(x)
}

#' Is this a gtable?
#' 
#' @param x object to test
#' @export
is.gtable <- function(x) {
  inherits(x, "gtable")
}

#' @S3method t gtable
t.gtable <- function(x) {
  new <- x
  
  new$layout$t <- x$layout$l
  new$layout$r <- x$layout$b
  new$layout$b <- x$layout$r
  new$layout$l <- x$layout$t
  
  new$widths <- x$heights
  new$heights <- x$widths
  
  new
}

#' @S3method [ gtable
"[.gtable" <- function(x, i, j) {
  # Convert indicies to (named) numeric
  rows <- setNames(seq_along(x$heights), rownames(x))[i]
  cols <- setNames(seq_along(x$widths), colnames(x))[j]

  i <- seq_along(x$heights) %in% seq_along(x$heights)[rows]
  j <- seq_along(x$widths) %in% seq_along(x$widths)[cols]  
  
  x$heights <- x$heights[rows]
  x$rownames <- x$rownames[rows]
  x$widths <- x$widths[cols]
  x$colnames <- x$colnames[cols]
  
  keep <- x$layout$t %in% rows & x$layout$b %in% rows & 
          x$layout$l %in% cols & x$layout$r %in% cols
  x$grobs <- x$grobs[keep]
  
  adj_rows <- cumsum(!i)
  adj_cols <- cumsum(!j)
  
  x$layout$r <- x$layout$r - adj_cols[x$layout$r]
  x$layout$l <- x$layout$l - adj_cols[x$layout$l]
  x$layout$t <- x$layout$t - adj_rows[x$layout$t]
  x$layout$b <- x$layout$b - adj_rows[x$layout$b]
  
  # Drop the unused rows from layout
  x$layout <- x$layout[keep, ]
  x
}

#' @S3method length gtable
length.gtable <- function(x) length(x$grobs)

#' Returns the height of a gtable, in the gtable's units
#'
#' Note that unlike heightDetails.gtable, this can return relative units.
#'
#' @param x A gtable object
#' @export
gtable_height <- function(x) sum(x$heights)

#' Returns the width of a gtable, in the gtable's units
#'
#' Note that unlike widthDetails.gtable, this can return relative units.
#'
#' @param x A gtable object
#' @export
gtable_width <- function(x) sum(x$widths)

#' Add new rows in specified position.
#'
#' Insert new rows in a gtable and adjust the grob placement accordingly. If
#' rows are added in the middle of a grob spanning multiple rows, the grob will
#' continue to span them all. If a row is added above or below a grob, the grob
#' will not span the new row(s).
#'
#' @param x a [gtable()] object
#' @param heights a unit vector giving the heights of the new rows
#' @param pos new row will be added below this position. Defaults to
#'   adding row on bottom. `0` adds on the top.
#'
#' @return A gtable with the new rows added.
#'
#' @family gtable manipulation
#'
#' @export
#'
#' @examples
#' library(grid)
#' rect <- rectGrob(gp = gpar(fill = "#00000080"))
#' tab <- gtable(unit(rep(1, 3), "null"), unit(rep(1, 3), "null"))
#' tab <- gtable_add_grob(tab, rect, t = 1, l = 1, r = 3)
#' tab <- gtable_add_grob(tab, rect, t = 1, b = 3, l = 1)
#' tab <- gtable_add_grob(tab, rect, t = 1, b = 3, l = 3)
#' dim(tab)
#' plot(tab)
#'
#' # Grobs will continue to span over new rows if added in the middle
#' tab2 <- gtable_add_rows(tab, unit(1, "null"), 1)
#' dim(tab2)
#' plot(tab2)
#'
#' # But not when added to top (0) or bottom (-1, the default)
#' tab3 <- gtable_add_rows(tab, unit(1, "null"))
#' tab3 <- gtable_add_rows(tab3, unit(1, "null"), 0)
#' dim(tab3)
#' plot(tab3)
#'
gtable_add_rows <- function(x, heights, pos = -1) {
  check_gtable(x)
  check_number_whole(pos)
  n <- length(heights)

  pos <- neg_to_pos(pos, length(x$heights))

  # Shift existing rows down
  x$heights <- insert.unit(x$heights, heights, pos)
  layout <- unclass(x$layout)
  layout$t <- ifelse(layout$t > pos, layout$t + n, layout$t)
  layout$b <- ifelse(layout$b > pos, layout$b + n, layout$b)
  x$layout <- new_data_frame(layout)

  x
}

#' Add new columns in specified position.
#'
#' Insert new columns in a gtable and adjust the grob placement accordingly. If
#' columns are added in the middle of a grob spanning multiple columns, the grob
#' will continue to span them all. If a column is added to the left or right of
#' a grob, the grob will not span the new column(s).
#'
#' @param x a [gtable()] object
#' @param widths a unit vector giving the widths of the new columns
#' @param pos new columns will be added to the right of this position. Defaults
#'   to adding col on right. `0` adds on the left.
#'
#' @return A gtable with the new columns added.
#'
#' @family gtable manipulation
#'
#' @export
#'
#' @examples
#' library(grid)
#' rect <- rectGrob(gp = gpar(fill = "#00000080"))
#' tab <- gtable(unit(rep(1, 3), "null"), unit(rep(1, 3), "null"))
#' tab <- gtable_add_grob(tab, rect, t = 1, l = 1, r = 3)
#' tab <- gtable_add_grob(tab, rect, t = 1, b = 3, l = 1)
#' tab <- gtable_add_grob(tab, rect, t = 1, b = 3, l = 3)
#' dim(tab)
#' plot(tab)
#'
#' # Grobs will continue to span over new rows if added in the middle
#' tab2 <- gtable_add_cols(tab, unit(1, "null"), 1)
#' dim(tab2)
#' plot(tab2)
#'
#' # But not when added to left (0) or right (-1, the default)
#' tab3 <- gtable_add_cols(tab, unit(1, "null"))
#' tab3 <- gtable_add_cols(tab3, unit(1, "null"), 0)
#' dim(tab3)
#' plot(tab3)
#'
gtable_add_cols <- function(x, widths, pos = -1) {
  check_gtable(x)
  check_number_whole(pos)
  n <- length(widths)

  pos <- neg_to_pos(pos, length(x$widths))

  # Shift existing columns right
  x$widths <- insert.unit(x$widths, widths, pos)
  layout <- unclass(x$layout)
  layout$l <- ifelse(layout$l > pos, layout$l + n, layout$l)
  layout$r <- ifelse(layout$r > pos, layout$r + n, layout$r)
  x$layout <- new_data_frame(layout)
  x
}

#' Row and column binding for gtables.
#'
#' @param ... gtables to combine (\code{x} and \code{y})
#' @param size How should the widths (for rbind) and the heights (for cbind)
#'   be combined across the gtables: take values from \code{first},
#'   or \code{last} gtable, or compute the \code{min} or \code{max} values.
#'   Defaults to \code{max}.
#' @param z A numeric vector indicating the relative z values of each gtable.
#'   The z values of each object in the resulting gtable will be modified
#'   to fit this order. If \code{NULL}, then the z values of obects within
#'   each gtable will not be modified.
#' @name bind
NULL

#' @rdname bind
#' @method rbind gtable
#' @export
rbind.gtable <- function(..., size = "max", z = NULL) {
  gtables <- list(...)
  if (!is.null(z)) {
    gtables <- z_arrange_gtables(gtables, z)
  }
  Reduce(function(x, y) rbind_gtable(x, y, size = size), gtables)
}

rbind_gtable <- function(x, y, size = "max") {
  stopifnot(ncol(x) == ncol(y))
  if (nrow(x) == 0) return(y)
  if (nrow(y) == 0) return(x)
  
  y$layout$t <- y$layout$t + nrow(x)
  y$layout$b <- y$layout$b + nrow(x)
  x$layout <- rbind(x$layout, y$layout)

  x$heights <- insert.unit(x$heights, y$heights)
  x$rownames <- c(x$rownames, y$rownames)

  size <- match.arg(size, c("first", "last", "max", "min"))
  x$widths <- switch(size,
    first = x$widths,
    last = y$widths,
    min = compare.unit(x$widths, y$widths, pmin),
    max = compare.unit(x$widths, y$widths, pmax)
  )

  x$grobs <- append(x$grobs, y$grobs)

  x
}

#' @rdname bind
#' @method cbind gtable
#' @export
cbind.gtable <- function(..., size = "max", z = NULL) {
  gtables <- list(...)
  if (!is.null(z)) {
    gtables <- z_arrange_gtables(gtables, z)
  }
  Reduce(function(x, y) cbind_gtable(x, y, size = size), gtables)
}

cbind_gtable <- function(x, y, size = "max") {
  stopifnot(nrow(x) == nrow(y))
  if (ncol(x) == 0) return(y)
  if (ncol(y) == 0) return(x)
  
  y$layout$l <- y$layout$l + ncol(x)
  y$layout$r <- y$layout$r + ncol(x)
  x$layout <- rbind(x$layout, y$layout)
  
  x$widths <- insert.unit(x$widths, y$widths)
  x$colnames <- c(x$colnames, y$colnames)

  size <- match.arg(size, c("first", "last", "max", "min"))
  x$heights <- switch(size,
    first = x$heights,
    last = y$heights,
    min = compare.unit(x$heights, y$heights, pmin),
    max = compare.unit(x$heights, y$heights, pmax)
  )

  x$grobs <- append(x$grobs, y$grobs)

  x
}

#' Filter cells by name.
#' 
#' @param x a gtable object
#' @inheritParams base::grepl
#' @param trim if \code{TRUE}, \code{\link{gtable_trim}} will be used to trim
#'   off any empty cells.
#' @export
#' @examples
#' gt <- gtable(unit(rep(5, 3), c("cm")), unit(5, "cm"))
#' rect <- rectGrob(gp = gpar(fill = "black"))
#' circ <- circleGrob(gp = gpar(fill = "red"))
#'
#' gt <- gtable_add_grob(gt, rect, 1, 1, name = "rect")
#' gt <- gtable_add_grob(gt, circ, 1, 3, name = "circ")
#'
#' plot(gtable_filter(gt, "rect"))
#' plot(gtable_filter(gt, "rect", trim = FALSE))
#' plot(gtable_filter(gt, "circ"))
#' plot(gtable_filter(gt, "circ", trim = FALSE))
gtable_filter <- function(x, pattern, fixed = FALSE, trim = TRUE) {
  
  matches <- grepl(pattern, x$layout$name, fixed = fixed)
  x$layout <- x$layout[matches, , drop = FALSE]
  x$grobs <- x$grobs[matches]
  
  if (trim) x <- gtable_trim(x)
  
  x
}

#' Normalise z values within a gtable object
#'
#' The z values within a gtable object can be any numeric values.
#' This function will change them to integers (starting from 1),
#' preserving the original order.
#'
#' Ties are handled by the \code{"first"} method: the first occurrence
#' of a value wins.
#'
#' @param x A gtable object
#' @param i The z value to start counting up from (default is 1)
z_normalise <- function(x, i = 1) {
  x$layout$z <- rank(x$layout$z, ties.method = "first") + i - 1
  x
}


#' Arrange the z values within gtable objects
#'
#' This is usually used before rbinding or cbinding the gtables together.
#' The resulting z values will be normalized.
#'
#' Ties are handled by the \code{"first"} method: the first occurrence
#' of a value wins.
#'
#' @param gtables A list of gtable objects
#' @param z A numeric vector of relative z values
z_arrange_gtables <- function(gtables, z) {
  if (length(gtables) != length(z)) {
    stop("'gtables' and 'z' must be the same length")
  }

  # Keep track of largest z value encountered so far
  zmax <- 0
  # Go through each gtable, in the order of z
  for (i in order(z)) {
    # max() gives a warning if zero-length input
    if (nrow(gtables[[i]]$layout) > 0) {
      gtables[[i]] <- z_normalise(gtables[[i]], zmax + 1)
      zmax <- max(gtables[[i]]$layout$z)
    }
  }

  gtables
}

#' @keywords internal
#' @aliases gtable-package
"_PACKAGE"

## usethis namespace: start
#' @import grid
#' @import rlang
#' @importFrom glue glue
#' @importFrom lifecycle deprecated
## usethis namespace: end
NULL

#' Create a single column gtable
#'
#' This function stacks a list of grobs into a single column gtable of the given
#' width and heights.
#'
#' @inheritParams gtable
#' @inheritParams gtable_add_grob
#' @param width a unit vector giving the width of this column
#' @param vp a grid viewport object (or NULL).
#'
#' @return A gtable with one column and as many rows as elements in the grobs
#' list.
#'
#' @family gtable construction
#'
#' @export
#'
#' @examples
#' library(grid)
#' a <- rectGrob(gp = gpar(fill = "red"))
#' b <- circleGrob()
#' c <- linesGrob()
#' gt <- gtable_col("demo", list(a, b, c))
#' gt
#' plot(gt)
#' gtable_show_layout(gt)
gtable_col <- function(name, grobs, width = NULL, heights = NULL,
                       z = NULL, vp = NULL, clip = "inherit") {
  width <- width %||% unit(max(unlist(lapply(grobs, width_cm))), "cm")
  heights <- heights %||% rep(unit(1, "null"), length(grobs))

  # z is either NULL, or a vector of the same length as grobs
  if (!(is.null(z) || length(z) == length(grobs))) {
    cli::cli_abort("{.arg z} must be either NULL or the same length as {.arg grobs}")
  }
  if (is.null(z)) {
    z <- Inf
  }

  table <- gtable(widths = width, heights = heights, name = name, vp = vp,
                  rownames = names(grobs))
  table <- gtable_add_grob(table, grobs,
    t = seq_along(grobs), l = 1,
    z = z, clip = clip
  )

  table
}

#' Create a single row gtable.
#'
#' This function puts grobs in a list side-by-side in a single-row gtable from
#' left to right witrh the given widths and height.
#'
#' @inheritParams gtable
#' @inheritParams gtable_add_grob
#' @param height a unit vector giving the height of this row
#' @param vp a grid viewport object (or NULL).
#'
#' @return A gtable with a single row and the same number of columns as
#' elements in the grobs list
#'
#' @family gtable construction
#'
#' @export
#'
#' @examples
#' library(grid)
#' a <- rectGrob(gp = gpar(fill = "red"))
#' b <- circleGrob()
#' c <- linesGrob()
#' gt <- gtable_row("demo", list(a, b, c))
#' gt
#' plot(gt)
#' gtable_show_layout(gt)
gtable_row <- function(name, grobs, height = NULL, widths = NULL,
                       z = NULL, vp = NULL, clip = "inherit") {
  height <- height %||% unit(max(unlist(lapply(grobs, height_cm))), "cm")
  widths <- widths %||% rep(unit(1, "null"), length(grobs))

  # z is either NULL, or a vector of the same length as grobs
  if (!(is.null(z) || length(z) == length(grobs))) {
    cli::cli_abort("{.arg z} must be either NULL or the same length as {.arg grobs}")
  }
  if (is.null(z)) {
    z <- Inf
  }

  table <- gtable(widths = widths, heights = height, name = name, vp = vp,
                  colnames = names(grobs))
  table <- gtable_add_grob(table, grobs,
    l = seq_along(grobs), t = 1,
    z = z, clip = clip
  )

  table
}

#' Create a gtable from a matrix of grobs.
#'
#' This function takes a matrix of grobs and create a gtable matching with the
#' grobs in the same position as they were in the matrix, with the given heights
#' and widths.
#'
#' @inheritParams gtable
#' @inheritParams gtable_add_grob
#' @param z a numeric matrix of the same dimensions as `grobs`,
#'   specifying the order that the grobs are drawn.
#' @param vp a grid viewport object (or NULL).
#'
#' @return A gtable of the same dimensions as the grobs matrix.
#'
#' @family gtable construction
#'
#' @export
#'
#' @examples
#' library(grid)
#' a <- rectGrob(gp = gpar(fill = "red"))
#' b <- circleGrob()
#' c <- linesGrob()
#'
#' row <- matrix(list(a, b, c), nrow = 1)
#' col <- matrix(list(a, b, c), ncol = 1)
#' mat <- matrix(list(a, b, c, nullGrob()), nrow = 2)
#'
#' gtable_matrix("demo", row, unit(c(1, 1, 1), "null"), unit(1, "null"))
#' gtable_matrix("demo", col, unit(1, "null"), unit(c(1, 1, 1), "null"))
#' gtable_matrix("demo", mat, unit(c(1, 1), "null"), unit(c(1, 1), "null"))
#'
#' # Can specify z ordering
#' z <- matrix(c(3, 1, 2, 4), nrow = 2)
#' gtable_matrix("demo", mat, unit(c(1, 1), "null"), unit(c(1, 1), "null"), z = z)
gtable_matrix <- function(name, grobs, widths = NULL, heights = NULL,
                          z = NULL, respect = FALSE, clip = "on", vp = NULL) {
  if (length(widths) != ncol(grobs)) {
    cli::cli_abort("{.arg widths} must be the same as the number of columns in {.arg grobs}")
  }
  if (length(heights) != nrow(grobs)) {
    cli::cli_abort("{.arg heights} must be the same as the number of rows in {.arg grobs}")
  }
  # z is either NULL or a matrix of the same dimensions as grobs
  if (!(is.null(z) || identical(dim(grobs), dim(z)))) {
    cli::cli_abort("{.arg z} must be either NULL or have the same dimensions as {.arg grobs}")
  }
  if (is.null(z)) {
    z <- Inf
  }
  table <- gtable(widths = widths, heights = heights, name = name,
                  respect = respect, vp = vp,
                  rownames = rownames(grobs), colnames = colnames(grobs))
  table <- gtable_add_grob(table, grobs,
    t = c(row(grobs)), l = c(col(grobs)),
    z = as.vector(z), clip = clip
  )

  table
}

#' Create a row/col spacer gtable.
#'
#' Create a zero-column or zero-row gtable with the given heights or widths
#' respectively.
#'
#' @name gtable_spacer
#'
#' @return A gtable object
#'
#' @family gtable construction
NULL

#' @param widths unit vector of widths
#' @rdname gtable_spacer
#' @export
gtable_row_spacer <- function(widths) {
  gtable_add_cols(gtable(), widths)
}

#' @param heights unit vector of heights
#' @rdname gtable_spacer
#' @export
gtable_col_spacer <- function(heights) {
  gtable_add_rows(gtable(), heights)
}

#' Add padding around edges of table.
#'
#' This is a convenience function for adding an extra row and an extra column at
#' each edge of the table.
#'
#' @param x a [gtable()] object
#' @param padding vector of length 4: top, right, bottom, left.  Normal
#'  recycling rules apply.
#'
#' @return A gtable object
#'
#' @family gtable manipulation
#'
#' @export
#'
#' @examples
#' library(grid)
#' gt <- gtable(unit(1, "null"), unit(1, "null"))
#' gt <- gtable_add_grob(gt, rectGrob(gp = gpar(fill = "black")), 1, 1)
#'
#' plot(gt)
#' plot(cbind(gt, gt))
#' plot(rbind(gt, gt))
#'
#' pad <- gtable_add_padding(gt, unit(1, "cm"))
#' plot(pad)
#' plot(cbind(pad, pad))
#' plot(rbind(pad, pad))
gtable_add_padding <- function(x, padding) {
  padding <- rep(padding, length.out = 4)

  x <- gtable_add_rows(x, pos = 0, heights = padding[1])
  x <- gtable_add_cols(x, pos = -1, widths = padding[2])
  x <- gtable_add_rows(x, pos = -1, heights = padding[3])
  x <- gtable_add_cols(x, pos = 0, widths = padding[4])
  x
}

check_gtable <- function(x,
                         ...,
                         allow_null = FALSE,
                         arg = caller_arg(x),
                         call = caller_env()) {
  if (!missing(x)) {
    if (is.gtable(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a gtable object",
    ...,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

check_unit <- function(x,
                       ...,
                       allow_null = FALSE,
                       arg = caller_arg(x),
                       call = caller_env()) {
  if (!missing(x)) {
    if (is.unit(x)) {
      return(invisible(NULL))
    }
    if (allow_null && is_null(x)) {
      return(invisible(NULL))
    }
  }

  stop_input_type(
    x,
    "a unit vector",
    ...,
    allow_null = allow_null,
    arg = arg,
    call = call
  )
}

neg_to_pos <- function(x, max) {
  ifelse(x >= 0, x, max + 1 + x)
}

compare_unit <- function(x, y, comp = `=`) {
  if (length(y) == 0) return(x)
  if (length(x) == 0) return(y)
  if (getRversion() >= "3.6" && (is.list(x) || is.list(y))) {
    if (identical(comp, pmin)) {
      return(unit.pmin(x, y))
    }
    if (identical(comp, pmax)) {
      return(unit.pmax(x, y))
    }
    cli::cli_abort('comparison not supported')
  }
  # Below should be removed once the old grid unit implementation has been deprecated
  x_attr <- attributes(x)
  x_val <- unclass(x)
  y_val <- unclass(y)

  x_unit <- x_attr$unit
  y_unit <- attr(x, "unit")

  if (!all(x_unit == y_unit)) {
    cli::cli_abort("comparison of units with different types currently not supported")
  }

  `attributes<-`(comp(x_val, y_val), x_attr)
}


insert.unit <- function(x, values, after = length(x)) {
  lengx <- length(x)
  if (lengx == 0) return(values)
  if (length(values) == 0) return(x)

  if (after <= 0) {
    unit.c(values, x)
  } else if (after >= lengx) {
    unit.c(x, values)
  } else {
    unit.c(x[1L:after], values, x[(after + 1L):lengx])
  }
}

"%||%" <- function(a, b) {
  if (!is.null(a)) a else b
}

width_cm <- function(x) {
  if (is.grob(x)) {
    convertWidth(grobWidth(x), "cm", TRUE)
  } else if (is.list(x)) {
    vapply(x, width_cm, numeric(1))
  } else if (is.unit(x)) {
    convertWidth(x, "cm", TRUE)
  } else {
    cli::cli_abort("Unknown input")
  }
}
height_cm <- function(x) {
  if (is.grob(x)) {
    convertWidth(grobHeight(x), "cm", TRUE)
  } else if (is.list(x)) {
    vapply(x, height_cm, numeric(1))
  } else if (is.unit(x)) {
    convertHeight(x, "cm", TRUE)
  } else {
    cli::cli_abort("Unknown input")
  }
}

# Check that x is same length as g, or length 1
len_same_or_1 <- function(x, n) {
  length(x) == 1 || length(x) == n
}

#' Trim off empty cells.
#'
#' This function detects rows and columns that does not contain any grobs and
#' removes thewm from the gtable. If the rows and/or columns removed had a
#' non-zero height/width the relative layout of the gtable may change.
#'
#' @param x a gtable object
#'
#' @return A gtable object
#'
#' @family gtable modification
#'
#' @export
#'
#' @examples
#' library(grid)
#' rect <- rectGrob(gp = gpar(fill = "black"))
#' base <- gtable(unit(c(2, 2, 2), "cm"), unit(c(2, 2, 2), "cm"))
#'
#' center <- gtable_add_grob(base, rect, 2, 2)
#' plot(center)
#' plot(gtable_trim(center))
#'
#' col <- gtable_add_grob(base, rect, 1, 2, 3, 2)
#' plot(col)
#' plot(gtable_trim(col))
#'
#' row <- gtable_add_grob(base, rect, 2, 1, 2, 3)
#' plot(row)
#' plot(gtable_trim(row))
gtable_trim <- function(x) {
  check_gtable(x)
  if (length(x) == 0) {
    return(gtable(respect = x$respect, name = x$name, vp = x$vp))
  }

  layout <- unclass(x$layout)

  w <- range(layout$l, layout$r)
  h <- range(layout$t, layout$b)

  x$widths <- x$widths[seq.int(w[1], w[2])]
  x$heights <- x$heights[seq.int(h[1], h[2])]

  layout$l <- layout$l - w[1] + 1
  layout$r <- layout$r - w[1] + 1
  layout$t <- layout$t - h[1] + 1
  layout$b <- layout$b - h[1] + 1

  x$layout <- new_data_frame(layout)

  x
}

#' Create a new grob table.
#'
#' A grob table captures all the information needed to layout grobs in a table
#' structure. It supports row and column spanning, offers some tools to
#' automatically figure out the correct dimensions, and makes it easy to
#' align and combine multiple tables.
#'
#' Each grob is put in its own viewport - grobs in the same location are
#' not combined into one cell. Each grob takes up the entire cell viewport
#' so justification control is not available.
#'
#' It constructs both the viewports and the gTree needed to display the table.
#'
#' @section Components:
#'
#' There are three basics components to a grob table: the specification of
#' table (cell heights and widths), the layout (for each grob, its position,
#' name and other settings), and global parameters.
#'
#' It's easier to understand how `gtable` works if in your head you keep
#' the table separate from it's contents.  Each cell can have 0, 1, or many
#' grobs inside. Each grob must belong to at least one cell, but can span
#' across many cells.
#'
#' @section Layout:
#'
#' The layout details are stored in a data frame with one row for each grob,
#' and columns:
#'
#' \itemize{
#'   \item `t` top extent of grob
#'   \item `r` right extent of grob
#'   \item `b` bottom extent of
#'   \item `l` left extent of grob
#'   \item `z` the z-order of the grob - used to reorder the grobs
#'     before they are rendered
#'   \item `clip` a string, specifying how the grob should be clipped:
#'     either `"on"`, `"off"` or `"inherit"`
#'   \item `name`, a character vector used to name each grob and its
#'     viewport
#' }
#'
#' You should not need to modify this data frame directly - instead use
#' functions like `gtable_add_grob`.
#'
#' @param widths a unit vector giving the width of each column
#' @param heights a unit vector giving the height of each row
#' @param respect a logical vector of length 1: should the aspect ratio of
#'   height and width specified in null units be respected.  See
#'   [grid.layout()] for more details
#' @param name a string giving the name of the table. This is used to name
#'   the layout viewport
#' @param rownames,colnames character vectors of row and column names, used
#'   for characteric subsetting.
#' @param vp a grid viewport object (or NULL).
#'
#' @return A gtable object
#'
#' @family gtable construction
#'
#' @export
#'
#' @examples
#' library(grid)
#' a <- gtable(unit(1:3, c("cm")), unit(5, "cm"))
#' a
#' gtable_show_layout(a)
#'
#' # Add a grob:
#' rect <- rectGrob(gp = gpar(fill = "black"))
#' a <- gtable_add_grob(a, rect, 1, 1)
#' a
#' plot(a)
#'
#' # gtables behave like matrices:
#' dim(a)
#' t(a)
#' plot(t(a))
#'
#' # when subsetting, grobs are retained if their extents lie in the
#' # rows/columns that retained.
#'
#' b <- gtable(unit(c(2, 2, 2), "cm"), unit(c(2, 2, 2), "cm"))
#' b <- gtable_add_grob(b, rect, 2, 2)
#' b[1, ]
#' b[, 1]
#' b[2, 2]
#'
#' # gtable have row and column names
#' rownames(b) <- 1:3
#' rownames(b)[2] <- 200
#' colnames(b) <- letters[1:3]
#' dimnames(b)
gtable <- function(widths = list(), heights = list(), respect = FALSE,
                   name = "layout", rownames = NULL, colnames = NULL, vp = NULL) {
  if (length(widths) > 0) {
    check_unit(widths)
    if (!(is.null(colnames) || length(colnames == length(widths)))) {
      cli::cli_abort("{.arg colnames} must either be NULL or have the same length as {.arg widths}")
    }
  }
  if (length(heights) > 0) {
    check_unit(heights)
    if (!(is.null(rownames) || length(rownames == length(heights)))) {
      cli::cli_abort("{.arg rownames} must either be NULL or have the same length as {.arg heights}")
    }
  }

  layout <- new_data_frame(list(
    t = numeric(), l = numeric(), b = numeric(), r = numeric(), z = numeric(),
    clip = character(), name = character()
  ), n = 0)

  if (!is.null(vp)) {
    vp <- viewport(
      name = name,
      x = vp$x, y = vp$y,
      width = vp$width, height = vp$height,
      just = vp$justification, gp = vp$gp, xscale = vp$xscale,
      yscale = vp$yscale, angle = vp$angle, clip = vp$clip
    )
  }

  gTree(
    grobs = list(), layout = layout, widths = widths,
    heights = heights, respect = respect, name = name,
    rownames = rownames, colnames = colnames, vp = vp,
    cl = "gtable"
  )
}

#' Print a gtable object
#'
#' @param x A gtable object.
#' @param zsort Sort by z values? Default `FALSE`.
#' @param ... Other arguments (not used by this method).
#' @export
#' @method print gtable
print.gtable <- function(x, zsort = FALSE, ...) {
  cat("TableGrob (", length(x$heights), " x ", length(x$widths), ") \"", x$name, "\": ",
      length(x$grobs), " grobs\n", sep = "")

  if (nrow(x$layout) == 0) return()

  pos <- as.data.frame(format(as.matrix(x$layout[c("t", "r", "b", "l")])),
    stringsAsFactors = FALSE
  )
  grobNames <- vapply(x$grobs, as.character, character(1))

  info <- data.frame(
    z = x$layout$z,
    cells = paste("(", pos$t, "-", pos$b, ",", pos$l, "-", pos$r, ")", sep = ""),
    name = x$layout$name,
    grob = grobNames
  )
  if (zsort) info <- info[order(x$layout$z), ]

  print(info)
}


#' @export
dim.gtable <- function(x) c(length(x$heights), length(x$widths))

#' @export
dimnames.gtable <- function(x, ...) list(x$rownames, x$colnames)

#' @export
"dimnames<-.gtable" <- function(x, value) {
  x$rownames <- value[[1]]
  x$colnames <- value[[2]]

  if (anyDuplicated(x$rownames)) {
    cli::cli_abort("rownames must be distinct")
  }
  if (anyDuplicated(x$colnames)) {
    cli::cli_abort("colnames must be distinct")
  }

  x
}

#' @export
plot.gtable <- function(x, bg = NULL, grill = NULL, ...) {
  grid.newpage()
  if (!is.null(bg)) {
    grid.rect(gp = gpar(fill = bg))
  }
  if (!is.null(grill)) {
    grid <- seq(0, 1, length.out = 20)
    grid.grill(h = grid, v = grid, gp = gpar(col = grill))
  }
  grid.draw(x)
}

#' Is this a gtable?
#'
#' @param x object to test
#' @export
is.gtable <- function(x) {
  inherits(x, "gtable")
}

#' @export
t.gtable <- function(x) {
  new <- x
  layout <- unclass(x$layout)
  old_lay <- layout

  layout$t <- old_lay$l
  layout$r <- old_lay$b
  layout$b <- old_lay$r
  layout$l <- old_lay$t

  new$layout <- new_data_frame(layout)
  new$widths <- x$heights
  new$heights <- x$widths

  new
}

#' @export
"[.gtable" <- function(x, i, j) {
  # Convert indicies to (named) numeric
  rows <- stats::setNames(seq_along(x$heights), rownames(x))[i]
  cols <- stats::setNames(seq_along(x$widths), colnames(x))[j]

  rows_cur <- stats::na.omit(rows)
  cols_cur <- stats::na.omit(cols)

  if ((length(rows) > 1 && any(diff(rows_cur) < 1)) ||
      (length(cols) > 1 && any(diff(cols_cur) < 1))) {
    cli::cli_abort("{.arg i} and {.arg j} must be increasing sequences of numbers")
  }

  i <- seq_along(x$heights) %in% seq_along(x$heights)[rows_cur]
  j <- seq_along(x$widths) %in% seq_along(x$widths)[cols_cur]

  x$heights <- x$heights[rows_cur]
  x$rownames <- x$rownames[rows_cur]
  x$widths <- x$widths[cols_cur]
  x$colnames <- x$colnames[cols_cur]

  layout <- unclass(x$layout)

  keep <- layout$t %in% rows_cur & layout$b %in% rows_cur &
          layout$l %in% cols_cur & layout$r %in% cols_cur
  x$grobs <- x$grobs[keep]

  adj_rows <- cumsum(!stats::na.omit(i))
  adj_cols <- cumsum(!stats::na.omit(j))

  layout$r <- layout$r - adj_cols[layout$r]
  layout$l <- layout$l - adj_cols[layout$l]
  layout$t <- layout$t - adj_rows[layout$t]
  layout$b <- layout$b - adj_rows[layout$b]

  # Drop the unused rows from layout
  x$layout <- new_data_frame(layout)[keep, ]

  if (anyNA(rows)) {
    for (i in which(is.na(rows))) {
      x <- gtable_add_rows(x, unit(0, "mm"), i - 1)
    }
  }
  if (anyNA(cols)) {
    for (i in which(is.na(cols))) {
      x <- gtable_add_cols(x, unit(0, "mm"), i - 1)
    }
  }
  x
}

#' @export
length.gtable <- function(x) length(x$grobs)

#' Returns the height of a gtable, in the gtable's units
#'
#' Note that unlike heightDetails.gtable, this can return relative units.
#'
#' @param x A gtable object
#' @export
gtable_height <- function(x) sum(x$heights)

#' Returns the width of a gtable, in the gtable's units
#'
#' Note that unlike widthDetails.gtable, this can return relative units.
#'
#' @param x A gtable object
#' @export
gtable_width <- function(x) sum(x$widths)

#' Convert to a gtable
#'
#' @param x An object to convert.
#' @param ... Arguments forwarded to methods.
#'
#' @return A gtable object
#' @export
as.gtable <- function(x, ...) {
  check_dots_used()
  UseMethod("as.gtable")
}

#' @export
as.gtable.default <- function(x, ...) {
  cli::cli_abort("Can't convert {.obj_type_friendly {x}} to a {.cls gtable}.")
}

#' @export
as.gtable.gtable <- function(x, ...) x

#' @export
#' @describeIn as.gtable Creates a 1-cell gtable containing the grob.
#' @param widths,heights Scalar unit setting the size of the table. Defaults
#'   to [grid::grobWidth()] and [grid::grobHeight()] of `x` respectively.
as.gtable.grob <- function(x, widths = NULL, heights = NULL, ...) {
  if (length(widths) > 1) {
    widths <- widths[1]
    cli::cli_warn("{.arg widths} truncated to length 1.")
  }
  if (length(heights) > 1) {
    heights <- heights[1]
    cli::cli_warn("{.arg heights} truncated to length 1.")
  }
  table <- gtable(
    widths  = widths  %||% grobWidth(x),
    heights = heights %||% grobHeight(x)
  )
  gtable_add_grob(table, x, t = 1L, l = 1L, b = 1L, r = 1L, ...)
}

test_that("as.gtable sensibly converts objects", {

  # gtable --> gtable is a no-op
  g1 <- gtable(unit(1, "npc"), unit(1, "npc"))
  g2 <- circleGrob(r = unit(1, "cm"))

  expect_identical(as.gtable(g1), g1)

  test <- as.gtable(g2)
  expect_s3_class(test, "gtable")
  expect_equal(as.numeric(convertUnit(gtable_width(test), "cm")), 2)
  expect_equal(as.numeric(convertUnit(gtable_height(test), "cm")), 2)

  expect_snapshot(as.gtable(g2, widths = unit(c(1, 1), "cm")))
  expect_snapshot(as.gtable(g2, heights = unit(c(1, 1), "cm")))

  expect_snapshot(as.gtable(1:5), error = TRUE)
  expect_snapshot(as.gtable(g1, foo = "bar"), error = TRUE)
})

#' Row and column binding for gtables.
#'
#' These functions are the parallels of the `matrix`/`data.frame` row and
#' column bindings. As such they work in the same way, except they have to take
#' care of additional attributes within the gtables. Most importantly it needs
#' to take care of the sizing of the final gtable, as the different gtables
#' going in may have different widths or heights. By default it tries to
#' calculate the maximum width/height among the supplied gtables, but other
#' options exists. Further, the relative layering of the grobs in each gtable
#' can be modified or left as-is.
#'
#' @param ... gtables to combine (`x` and `y`)
#' @param size How should the widths (for rbind) and the heights (for cbind)
#'   be combined across the gtables: take values from `first`,
#'   or `last` gtable, or compute the `min` or `max` values.
#'   Defaults to `max`.
#' @param z A numeric vector indicating the relative z values of each gtable.
#'   The z values of each object in the resulting gtable will be modified
#'   to fit this order. If `NULL`, then the z values of obects within
#'   each gtable will not be modified.
#'
#' @return A gtable object
#'
#' @name bind
#'
#' @examples
#' library(grid)
#' a <- rectGrob(gp = gpar(fill = "red"))
#' b <- circleGrob()
#' c <- linesGrob()
#'
#' row <- matrix(list(a, b), nrow = 1)
#' col <- matrix(list(a, b), ncol = 1)
#' mat <- matrix(list(a, b, c, nullGrob()), nrow = 2)
#'
#' row_gt <- gtable_matrix("demo", row, unit(c(1, 1), "null"), unit(1, "null"))
#' col_gt <- gtable_matrix("demo", col, unit(1, "null"), unit(c(1, 1), "null"))
#' mat_gt <- gtable_matrix("demo", mat, unit(c(1, 1), "null"), unit(c(1, 1), "null"))
#'
#' # cbind
#' c_binded <- cbind(mat_gt, col_gt, size = "first")
#' plot(c_binded)
#'
#' # rbind
#' r_binded <- rbind(mat_gt, row_gt, size = "last")
#' plot(r_binded)
#'
#' # Dimensions must match along bind direction
#' try(cbind(mat_gt, row_gt))
#'
NULL

#' @rdname bind
#' @method rbind gtable
#' @export
rbind.gtable <- function(..., size = "max", z = NULL) {
  gtables <- list(...)
  if (!is.null(z)) {
    gtables <- z_arrange_gtables(gtables, z)
  }
  Reduce(function(x, y) rbind_gtable(x, y, size = size), gtables)
}

rbind_gtable <- function(x, y, size = "max") {
  if (length(x$widths) != length(y$widths)) {
    cli::cli_abort("{.arg x} and {.arg y} must have the same number of columns")
  }
  x_row <- length(x$heights)
  y_row <- length(y$heights)
  if (x_row == 0) return(y)
  if (y_row == 0) return(x)

  lay_x <- unclass(x$layout)
  lay_y <- unclass(y$layout)

  x$layout <- new_data_frame(list(
    t = c(lay_x$t, lay_y$t + x_row),
    l = c(lay_x$l, lay_y$l),
    b = c(lay_x$b, lay_y$b + x_row),
    r = c(lay_x$r, lay_y$r),
    z = c(lay_x$z, lay_y$z),
    clip = c(lay_x$clip, lay_y$clip),
    name = c(lay_x$name, lay_y$name)
  ))

  x$heights <- insert.unit(x$heights, y$heights)
  x$rownames <- c(x$rownames, y$rownames)

  size <- match.arg(size, c("first", "last", "max", "min"))
  x$widths <- switch(size,
    first = x$widths,
    last = y$widths,
    min = compare_unit(x$widths, y$widths, pmin),
    max = compare_unit(x$widths, y$widths, pmax)
  )

  x$grobs <- append(x$grobs, y$grobs)

  x
}

#' @rdname bind
#' @method cbind gtable
#' @export
cbind.gtable <- function(..., size = "max", z = NULL) {
  gtables <- list(...)
  if (!is.null(z)) {
    gtables <- z_arrange_gtables(gtables, z)
  }
  Reduce(function(x, y) cbind_gtable(x, y, size = size), gtables)
}

cbind_gtable <- function(x, y, size = "max") {
  if (length(x$heights) != length(y$heights)) {
    cli::cli_abort("{.arg x} and {.arg y} must have the same number of rows")
  }
  x_col <- length(x$widths)
  y_col <- length(y$widths)
  if (x_col == 0) return(y)
  if (y_col == 0) return(x)

  lay_x <- unclass(x$layout)
  lay_y <- unclass(y$layout)

  x$layout <- new_data_frame(list(
    t = c(lay_x$t, lay_y$t),
    l = c(lay_x$l, lay_y$l + x_col),
    b = c(lay_x$b, lay_y$b),
    r = c(lay_x$r, lay_y$r + x_col),
    z = c(lay_x$z, lay_y$z),
    clip = c(lay_x$clip, lay_y$clip),
    name = c(lay_x$name, lay_y$name)
  ))

  x$widths <- insert.unit(x$widths, y$widths)
  x$colnames <- c(x$colnames, y$colnames)

  size <- match.arg(size, c("first", "last", "max", "min"))
  x$heights <- switch(size,
    first = x$heights,
    last = y$heights,
    min = compare_unit(x$heights, y$heights, pmin),
    max = compare_unit(x$heights, y$heights, pmax)
  )

  x$grobs <- append(x$grobs, y$grobs)

  x
}

#' Filter cells by name
#'
#' Normally a gtable is considered a matrix when indexing so that indexing is
#' working on the cell layout and not on the grobs it contains. `gtable_filter`
#' allows you to subset the grobs by name and optionally remove rows or columns
#' if left empty after the subsetting
#'
#' @param x a gtable object
#' @inheritParams base::grepl
#' @param trim if `TRUE`, [gtable_trim()] will be used to trim
#'   off any empty cells.
#' @param invert Should the filtering be inverted so that cells matching
#'   `pattern` is removed instead of kept.
#'
#' @return A gtable only containing the matching grobs, potentially stripped of
#' empty columns and rows
#'
#' @family gtable manipulation
#'
#' @export
#'
#' @examples
#' library(grid)
#' gt <- gtable(unit(rep(5, 3), c("cm")), unit(5, "cm"))
#' rect <- rectGrob(gp = gpar(fill = "black"))
#' circ <- circleGrob(gp = gpar(fill = "red"))
#'
#' gt <- gtable_add_grob(gt, rect, 1, 1, name = "rect")
#' gt <- gtable_add_grob(gt, circ, 1, 3, name = "circ")
#'
#' plot(gtable_filter(gt, "rect"))
#' plot(gtable_filter(gt, "rect", trim = FALSE))
#' plot(gtable_filter(gt, "circ"))
#' plot(gtable_filter(gt, "circ", trim = FALSE))
#'
gtable_filter <- function(x, pattern, fixed = FALSE, trim = TRUE, invert = FALSE) {
  matches <- grepl(pattern, .subset2(x$layout, "name"), fixed = fixed)
  if (invert) matches <- !matches
  x$layout <- x$layout[matches, , drop = FALSE]
  x$grobs <- x$grobs[matches]

  if (trim) x <- gtable_trim(x)

  x
}

---
title: "Profiling Performance"
author: "Thomas Lin Pedersen"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Profiling Performance}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
#| label: setup
#| include: false
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

In order to continuously monitor the performance of gtable
the following piece of code is used to generate a profile and inspect it:

```{r}
library(ggplot2)
library(profvis)

p <- ggplot(mtcars, aes(mpg, disp)) + 
  geom_point() + 
  facet_grid(gear~cyl)

p_build <- ggplot_build(p)

profile <- profvis(for (i in seq_len(100)) ggplot_gtable(p_build))

profile
```

```{r}
#| eval: false
#| include: false
saveRDS(profile, file.path('profilings', paste0(packageVersion('gtable'), '.rds')))
```

The use of an empty ggplot2 ensures that the profile is based on real-life
use and includes complex gtable assembly. Profiles for old version are kept 
for reference and can be accessed at the 
[github repository](https://github.com/r-lib/gtable/tree/master/vignettes/profilings).
Care should be taken in not comparing profiles across versions, as 
changes to code outside of gtable can have profound effect on the results.
Thus, the intend of profiling is to identify bottlenecks in the implementation
that are ripe for improvement, more then to quantify improvements to performance
over time.

## Performance focused changes across versions
To keep track of changes focused on improving the performance of gtable they
are summarised below:

### v`r packageVersion('gtable')`
Profiling results from gtable v0.2.0 identified a range of areas that could be
easily improved by fairly small code changes. These changes resulted in roughly
20% decrease in running time on the profiling code in general, while gtable 
related functions were between 50 and 80% decrease in running time specifically.

- **`data.frame` construction and indexing.** gtable now includes a minimal 
  constructor that makes no input checking used for working with the layout data
  frame. Further, indexing into the layout data frame has been improved by 
  either treating as a list internally or directly calling `.subset2`
- **Input validation.** `stopifnot()` was identified as a bottleneck and has 
  removed in favor of a standard `if (...) stop()`
- **Dimension querying.** The use of `nrow()` and `ncol()` has internally been 
  substituted for direct calls to `length()` of the `heights` and `widths` unit
  vectors

# as.gtable sensibly converts objects

    Code
      as.gtable(g2, widths = unit(c(1, 1), "cm"))
    Condition
      Warning:
      `widths` truncated to length 1.
    Output
      TableGrob (1 x 1) "layout": 1 grobs
        z     cells   name                  grob
      1 1 (1-1,1-1) layout circle[GRID.circle.9]

---

    Code
      as.gtable(g2, heights = unit(c(1, 1), "cm"))
    Condition
      Warning:
      `heights` truncated to length 1.
    Output
      TableGrob (1 x 1) "layout": 1 grobs
        z     cells   name                  grob
      1 1 (1-1,1-1) layout circle[GRID.circle.9]

---

    Code
      as.gtable(1:5)
    Condition
      Error in `as.gtable()`:
      ! Can't convert an integer vector to a <gtable>.

---

    Code
      as.gtable(g1, foo = "bar")
    Condition
      Error in `as.gtable()`:
      ! Arguments in `...` must be used.
      x Problematic argument:
      * foo = "bar"
      i Did you misspell an argument name?

# indexing errors are caught

    Code
      base[2:1, ]
    Condition
      Error in `base[2:1, ]`:
      ! `i` and `j` must be increasing sequences of numbers

---

    Code
      base[c(2, 2), ]
    Condition
      Error in `base[c(2, 2), ]`:
      ! `i` and `j` must be increasing sequences of numbers

---

    Code
      base[, 2:1]
    Condition
      Error in `base[, 2:1]`:
      ! `i` and `j` must be increasing sequences of numbers

---

    Code
      base[, c(1, 1)]
    Condition
      Error in `base[, c(1, 1)]`:
      ! `i` and `j` must be increasing sequences of numbers

# Adding multiple grobs

    Code
      gtable_add_grob(layout, grobs, c(1:3), 1, 3, 3)
    Condition
      Error in `gtable_add_grob()`:
      ! Not all inputs have either length 1 or same length same as `grobs`

---

    Code
      gtable_add_grob(layout, grobs, tval, 1:2, 3, 3)
    Condition
      Error in `gtable_add_grob()`:
      ! Not all inputs have either length 1 or same length same as `grobs`

---

    Code
      gtable_add_grob(layout, grobs, tval, 1, 3, 3, z = 1:4)
    Condition
      Error in `gtable_add_grob()`:
      ! Not all inputs have either length 1 or same length same as `grobs`

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable.R
\name{print.gtable}
\alias{print.gtable}
\title{Print a gtable object}
\usage{
\method{print}{gtable}(x, zsort = FALSE, ...)
}
\arguments{
\item{x}{A gtable object.}

\item{zsort}{Sort by z values? Default \code{FALSE}.}

\item{...}{Other arguments (not used by this method).}
}
\description{
Print a gtable object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rbind-cbind.R
\name{bind}
\alias{bind}
\alias{rbind.gtable}
\alias{cbind.gtable}
\title{Row and column binding for gtables.}
\usage{
\method{rbind}{gtable}(..., size = "max", z = NULL)

\method{cbind}{gtable}(..., size = "max", z = NULL)
}
\arguments{
\item{...}{gtables to combine (\code{x} and \code{y})}

\item{size}{How should the widths (for rbind) and the heights (for cbind)
be combined across the gtables: take values from \code{first},
or \code{last} gtable, or compute the \code{min} or \code{max} values.
Defaults to \code{max}.}

\item{z}{A numeric vector indicating the relative z values of each gtable.
The z values of each object in the resulting gtable will be modified
to fit this order. If \code{NULL}, then the z values of obects within
each gtable will not be modified.}
}
\value{
A gtable object
}
\description{
These functions are the parallels of the \code{matrix}/\code{data.frame} row and
column bindings. As such they work in the same way, except they have to take
care of additional attributes within the gtables. Most importantly it needs
to take care of the sizing of the final gtable, as the different gtables
going in may have different widths or heights. By default it tries to
calculate the maximum width/height among the supplied gtables, but other
options exists. Further, the relative layering of the grobs in each gtable
can be modified or left as-is.
}
\examples{
library(grid)
a <- rectGrob(gp = gpar(fill = "red"))
b <- circleGrob()
c <- linesGrob()

row <- matrix(list(a, b), nrow = 1)
col <- matrix(list(a, b), ncol = 1)
mat <- matrix(list(a, b, c, nullGrob()), nrow = 2)

row_gt <- gtable_matrix("demo", row, unit(c(1, 1), "null"), unit(1, "null"))
col_gt <- gtable_matrix("demo", col, unit(1, "null"), unit(c(1, 1), "null"))
mat_gt <- gtable_matrix("demo", mat, unit(c(1, 1), "null"), unit(c(1, 1), "null"))

# cbind
c_binded <- cbind(mat_gt, col_gt, size = "first")
plot(c_binded)

# rbind
r_binded <- rbind(mat_gt, row_gt, size = "last")
plot(r_binded)

# Dimensions must match along bind direction
try(cbind(mat_gt, row_gt))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add-grob.R
\name{gtable_add_grob}
\alias{gtable_add_grob}
\title{Add a single grob, possibly spanning multiple rows or columns.}
\usage{
gtable_add_grob(
  x,
  grobs,
  t,
  l,
  b = t,
  r = l,
  z = Inf,
  clip = "on",
  name = x$name
)
}
\arguments{
\item{x}{a \code{\link[=gtable]{gtable()}} object}

\item{grobs}{a single grob or a list of grobs}

\item{t}{a numeric vector giving the top extent of the grobs}

\item{l}{a numeric vector giving the left extent of the grobs}

\item{b}{a numeric vector giving the bottom extent of the grobs}

\item{r}{a numeric vector giving the right extent of the grobs}

\item{z}{a numeric vector giving the order in which the grobs should be
plotted.  Use \code{Inf} (the default) to plot above or \code{-Inf}
below all existing grobs. By default positions are on the integers,
giving plenty of room to insert new grobs between existing grobs.}

\item{clip}{should drawing be clipped to the specified cells
(\code{"on"}), the entire table (\code{"inherit"}), or not at all
(\code{"off"})}

\item{name}{name of the grob - used to modify the grob name before it's
plotted.}
}
\value{
A gtable object with the new grob(s) added
}
\description{
This only adds grobs into the table - it doesn't affect the table layout in
any way.  In the gtable model, grobs always fill up the complete table
cell.  If you want custom justification you might need to define the grob
dimension in absolute units, or put it into another gtable that can then be
added to the gtable instead of the grob.
}
\examples{
library(grid)

gt <- gtable(widths = unit(c(1, 1), 'null'), heights = unit(c(1, 1), 'null'))
pts <- pointsGrob(x = runif(5), y = runif(5))

# Add a grob to a single cell (top-right cell)
gt <- gtable_add_grob(gt, pts, t = 1, l = 2)

# Add a grob spanning multiple cells
gt <- gtable_add_grob(gt, pts, t = 1, l = 1, b = 2)

plot(gt)

}
\seealso{
Other gtable manipulation: 
\code{\link{gtable_add_cols}()},
\code{\link{gtable_add_padding}()},
\code{\link{gtable_add_rows}()},
\code{\link{gtable_add_space}},
\code{\link{gtable_filter}()}
}
\concept{gtable manipulation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable.R
\name{is.gtable}
\alias{is.gtable}
\title{Is this a gtable?}
\usage{
is.gtable(x)
}
\arguments{
\item{x}{object to test}
}
\description{
Is this a gtable?
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add-rows-cols.R
\name{gtable_add_cols}
\alias{gtable_add_cols}
\title{Add new columns in specified position.}
\usage{
gtable_add_cols(x, widths, pos = -1)
}
\arguments{
\item{x}{a \code{\link[=gtable]{gtable()}} object}

\item{widths}{a unit vector giving the widths of the new columns}

\item{pos}{new columns will be added to the right of this position. Defaults
to adding col on right. \code{0} adds on the left.}
}
\value{
A gtable with the new columns added.
}
\description{
Insert new columns in a gtable and adjust the grob placement accordingly. If
columns are added in the middle of a grob spanning multiple columns, the grob
will continue to span them all. If a column is added to the left or right of
a grob, the grob will not span the new column(s).
}
\examples{
library(grid)
rect <- rectGrob(gp = gpar(fill = "#00000080"))
tab <- gtable(unit(rep(1, 3), "null"), unit(rep(1, 3), "null"))
tab <- gtable_add_grob(tab, rect, t = 1, l = 1, r = 3)
tab <- gtable_add_grob(tab, rect, t = 1, b = 3, l = 1)
tab <- gtable_add_grob(tab, rect, t = 1, b = 3, l = 3)
dim(tab)
plot(tab)

# Grobs will continue to span over new rows if added in the middle
tab2 <- gtable_add_cols(tab, unit(1, "null"), 1)
dim(tab2)
plot(tab2)

# But not when added to left (0) or right (-1, the default)
tab3 <- gtable_add_cols(tab, unit(1, "null"))
tab3 <- gtable_add_cols(tab3, unit(1, "null"), 0)
dim(tab3)
plot(tab3)

}
\seealso{
Other gtable manipulation: 
\code{\link{gtable_add_grob}()},
\code{\link{gtable_add_padding}()},
\code{\link{gtable_add_rows}()},
\code{\link{gtable_add_space}},
\code{\link{gtable_filter}()}
}
\concept{gtable manipulation}

\name{gtable_height}
\alias{gtable_height}
\title{Returns the height of a gtable, in the gtable's units}
\usage{
  gtable_height(x)
}
\arguments{
  \item{x}{A gtable object}
}
\description{
  Note that unlike heightDetails.gtable, this can return
  relative units.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable.R
\name{gtable}
\alias{gtable}
\title{Create a new grob table.}
\usage{
gtable(
  widths = list(),
  heights = list(),
  respect = FALSE,
  name = "layout",
  rownames = NULL,
  colnames = NULL,
  vp = NULL
)
}
\arguments{
\item{widths}{a unit vector giving the width of each column}

\item{heights}{a unit vector giving the height of each row}

\item{respect}{a logical vector of length 1: should the aspect ratio of
height and width specified in null units be respected.  See
\code{\link[=grid.layout]{grid.layout()}} for more details}

\item{name}{a string giving the name of the table. This is used to name
the layout viewport}

\item{rownames, colnames}{character vectors of row and column names, used
for characteric subsetting.}

\item{vp}{a grid viewport object (or NULL).}
}
\value{
A gtable object
}
\description{
A grob table captures all the information needed to layout grobs in a table
structure. It supports row and column spanning, offers some tools to
automatically figure out the correct dimensions, and makes it easy to
align and combine multiple tables.
}
\details{
Each grob is put in its own viewport - grobs in the same location are
not combined into one cell. Each grob takes up the entire cell viewport
so justification control is not available.

It constructs both the viewports and the gTree needed to display the table.
}
\section{Components}{


There are three basics components to a grob table: the specification of
table (cell heights and widths), the layout (for each grob, its position,
name and other settings), and global parameters.

It's easier to understand how \code{gtable} works if in your head you keep
the table separate from it's contents.  Each cell can have 0, 1, or many
grobs inside. Each grob must belong to at least one cell, but can span
across many cells.
}

\section{Layout}{


The layout details are stored in a data frame with one row for each grob,
and columns:

\itemize{
\item \code{t} top extent of grob
\item \code{r} right extent of grob
\item \code{b} bottom extent of
\item \code{l} left extent of grob
\item \code{z} the z-order of the grob - used to reorder the grobs
before they are rendered
\item \code{clip} a string, specifying how the grob should be clipped:
either \code{"on"}, \code{"off"} or \code{"inherit"}
\item \code{name}, a character vector used to name each grob and its
viewport
}

You should not need to modify this data frame directly - instead use
functions like \code{gtable_add_grob}.
}

\examples{
library(grid)
a <- gtable(unit(1:3, c("cm")), unit(5, "cm"))
a
gtable_show_layout(a)

# Add a grob:
rect <- rectGrob(gp = gpar(fill = "black"))
a <- gtable_add_grob(a, rect, 1, 1)
a
plot(a)

# gtables behave like matrices:
dim(a)
t(a)
plot(t(a))

# when subsetting, grobs are retained if their extents lie in the
# rows/columns that retained.

b <- gtable(unit(c(2, 2, 2), "cm"), unit(c(2, 2, 2), "cm"))
b <- gtable_add_grob(b, rect, 2, 2)
b[1, ]
b[, 1]
b[2, 2]

# gtable have row and column names
rownames(b) <- 1:3
rownames(b)[2] <- 200
colnames(b) <- letters[1:3]
dimnames(b)
}
\seealso{
Other gtable construction: 
\code{\link{gtable_col}()},
\code{\link{gtable_matrix}()},
\code{\link{gtable_row}()},
\code{\link{gtable_spacer}}
}
\concept{gtable construction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter.R
\name{gtable_filter}
\alias{gtable_filter}
\title{Filter cells by name}
\usage{
gtable_filter(x, pattern, fixed = FALSE, trim = TRUE, invert = FALSE)
}
\arguments{
\item{x}{a gtable object}

\item{pattern}{character string containing a \link[base]{regular expression}
    (or character string for \code{fixed = TRUE}) to be matched
    in the given character vector.  Coerced by
    \code{\link[base]{as.character}} to a character string if possible.  If a
    character vector of length 2 or more is supplied, the first element
    is used with a warning.  Missing values are allowed except for
    \code{regexpr}, \code{gregexpr} and \code{regexec}.}

\item{fixed}{logical.  If \code{TRUE}, \code{pattern} is a string to be
    matched as is.  Overrides all conflicting arguments.}

\item{trim}{if \code{TRUE}, \code{\link[=gtable_trim]{gtable_trim()}} will be used to trim
off any empty cells.}

\item{invert}{Should the filtering be inverted so that cells matching
\code{pattern} is removed instead of kept.}
}
\value{
A gtable only containing the matching grobs, potentially stripped of
empty columns and rows
}
\description{
Normally a gtable is considered a matrix when indexing so that indexing is
working on the cell layout and not on the grobs it contains. \code{gtable_filter}
allows you to subset the grobs by name and optionally remove rows or columns
if left empty after the subsetting
}
\examples{
library(grid)
gt <- gtable(unit(rep(5, 3), c("cm")), unit(5, "cm"))
rect <- rectGrob(gp = gpar(fill = "black"))
circ <- circleGrob(gp = gpar(fill = "red"))

gt <- gtable_add_grob(gt, rect, 1, 1, name = "rect")
gt <- gtable_add_grob(gt, circ, 1, 3, name = "circ")

plot(gtable_filter(gt, "rect"))
plot(gtable_filter(gt, "rect", trim = FALSE))
plot(gtable_filter(gt, "circ"))
plot(gtable_filter(gt, "circ", trim = FALSE))

}
\seealso{
Other gtable manipulation: 
\code{\link{gtable_add_cols}()},
\code{\link{gtable_add_grob}()},
\code{\link{gtable_add_padding}()},
\code{\link{gtable_add_rows}()},
\code{\link{gtable_add_space}}
}
\concept{gtable manipulation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grid.R
\name{gtable_show_layout}
\alias{gtable_show_layout}
\title{Visualise the layout of a gtable.}
\usage{
gtable_show_layout(x, ...)
}
\arguments{
\item{x}{a gtable object}

\item{...}{
  Arguments passed on to \code{\link[grid:grid.show.layout]{grid::grid.show.layout}}
  \describe{
    \item{\code{l}}{A Grid layout object.}
    \item{\code{newpage}}{A logical value indicating whether to move on
    to a new page before drawing the diagram.}
    \item{\code{vp.ex}}{positive number, typically in \eqn{(0,1]}, specifying the
    scaling of the layout.}
    \item{\code{bg}}{The colour used for the background.}
    \item{\code{cell.border}}{The colour used to draw the borders of the cells
    in the layout.}
    \item{\code{cell.fill}}{The colour used to fill the cells in the layout.}
    \item{\code{cell.label}}{A logical indicating whether the layout cells
    should be labelled.}
    \item{\code{label.col}}{The colour used for layout cell labels.}
    \item{\code{unit.col}}{The colour used for labelling the widths/heights of
    columns/rows.}
    \item{\code{vp}}{A Grid viewport object (or NULL).}
  }}
}
\description{
This function is a simple wrapper around \code{\link[grid:grid.show.layout]{grid::grid.show.layout()}} that
allows you to inspect the layout of the gtable.
}
\examples{
gt <- gtable(widths = grid::unit(c(1, 0.5, 2), c("null", "cm", "null")),
             heights = grid::unit(c(0.2, 1, 3), c("inch", "null", "cm")))
gtable_show_layout(gt)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable.R
\name{as.gtable}
\alias{as.gtable}
\alias{as.gtable.grob}
\title{Convert to a gtable}
\usage{
as.gtable(x, ...)

\method{as.gtable}{grob}(x, widths = NULL, heights = NULL, ...)
}
\arguments{
\item{x}{An object to convert.}

\item{...}{Arguments forwarded to methods.}

\item{widths, heights}{Scalar unit setting the size of the table. Defaults
to \code{\link[grid:grobWidth]{grid::grobWidth()}} and \code{\link[grid:grobWidth]{grid::grobHeight()}} of \code{x} respectively.}
}
\value{
A gtable object
}
\description{
Convert to a gtable
}
\section{Methods (by class)}{
\itemize{
\item \code{as.gtable(grob)}: Creates a 1-cell gtable containing the grob.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add-space.R
\name{gtable_add_space}
\alias{gtable_add_space}
\alias{gtable_add_col_space}
\alias{gtable_add_row_space}
\title{Add row/column spacing.}
\usage{
gtable_add_col_space(x, width)

gtable_add_row_space(x, height)
}
\arguments{
\item{x}{a gtable object}

\item{width}{a vector of units of length 1 or ncol - 1}

\item{height}{a vector of units of length 1 or nrow - 1}
}
\value{
A gtable with the additional rows or columns added
}
\description{
Adds \code{width} space between the columns or \code{height} space between
the rows, effictvely pushing the existing cells apart.
}
\examples{
library(grid)

rect <- rectGrob()
rect_mat <- matrix(rep(list(rect), 9), nrow = 3)

gt <- gtable_matrix("rects", rect_mat, widths = unit(rep(1, 3), "null"),
                    heights = unit(rep(1, 3), "null"))

plot(gt)

# Add spacing between the grobs
# same height between all rows
gt <- gtable_add_row_space(gt, unit(0.5, "cm"))

# Different width between the columns
gt <- gtable_add_col_space(gt, unit(c(0.5, 1), "cm"))

plot(gt)
}
\seealso{
Other gtable manipulation: 
\code{\link{gtable_add_cols}()},
\code{\link{gtable_add_grob}()},
\code{\link{gtable_add_padding}()},
\code{\link{gtable_add_rows}()},
\code{\link{gtable_filter}()}
}
\concept{gtable manipulation}

\name{z_normalise}
\alias{z_normalise}
\title{Normalise z values within a gtable object}
\usage{
  z_normalise(x, i = 1)
}
\arguments{
  \item{x}{A gtable object}

  \item{i}{The z value to start counting up from (default
  is 1)}
}
\description{
  The z values within a gtable object can be any numeric
  values. This function will change them to integers
  (starting from 1), preserving the original order.
}
\details{
  Ties are handled by the \code{"first"} method: the first
  occurrence of a value wins.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable-package.R
\docType{package}
\name{gtable-package}
\alias{gtable-package}
\title{gtable: Arrange 'Grobs' in Tables}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Tools to make it easier to work with "tables" of 'grobs'. The 'gtable' package defines a 'gtable' grob class that specifies a grid along with a list of grobs and their placement in the grid. Further the package makes it easy to manipulate and combine 'gtable' objects so that complex compositions can be built up sequentially.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://gtable.r-lib.org}
  \item \url{https://github.com/r-lib/gtable}
  \item Report bugs at \url{https://github.com/r-lib/gtable/issues}
}

}
\author{
\strong{Maintainer}: Thomas Lin Pedersen \email{thomas.pedersen@posit.co}

Authors:
\itemize{
  \item Hadley Wickham \email{hadley@posit.co}
}

Other contributors:
\itemize{
  \item Posit Software, PBC [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable.R
\name{gtable_width}
\alias{gtable_width}
\title{Returns the width of a gtable, in the gtable's units}
\usage{
gtable_width(x)
}
\arguments{
\item{x}{A gtable object}
}
\description{
Note that unlike widthDetails.gtable, this can return relative units.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable-layouts.R
\name{gtable_spacer}
\alias{gtable_spacer}
\alias{gtable_row_spacer}
\alias{gtable_col_spacer}
\title{Create a row/col spacer gtable.}
\usage{
gtable_row_spacer(widths)

gtable_col_spacer(heights)
}
\arguments{
\item{widths}{unit vector of widths}

\item{heights}{unit vector of heights}
}
\value{
A gtable object
}
\description{
Create a zero-column or zero-row gtable with the given heights or widths
respectively.
}
\seealso{
Other gtable construction: 
\code{\link{gtable}()},
\code{\link{gtable_col}()},
\code{\link{gtable_matrix}()},
\code{\link{gtable_row}()}
}
\concept{gtable construction}

\name{z_arrange_gtables}
\alias{z_arrange_gtables}
\title{Arrange the z values within gtable objects}
\usage{
  z_arrange_gtables(gtables, z)
}
\arguments{
  \item{gtables}{A list of gtable objects}

  \item{z}{A numeric vector of relative z values}
}
\description{
  This is usually used before rbinding or cbinding the
  gtables together. The resulting z values will be
  normalized.
}
\details{
  Ties are handled by the \code{"first"} method: the first
  occurrence of a value wins.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/padding.R
\name{gtable_add_padding}
\alias{gtable_add_padding}
\title{Add padding around edges of table.}
\usage{
gtable_add_padding(x, padding)
}
\arguments{
\item{x}{a \code{\link[=gtable]{gtable()}} object}

\item{padding}{vector of length 4: top, right, bottom, left.  Normal
recycling rules apply.}
}
\value{
A gtable object
}
\description{
This is a convenience function for adding an extra row and an extra column at
each edge of the table.
}
\examples{
library(grid)
gt <- gtable(unit(1, "null"), unit(1, "null"))
gt <- gtable_add_grob(gt, rectGrob(gp = gpar(fill = "black")), 1, 1)

plot(gt)
plot(cbind(gt, gt))
plot(rbind(gt, gt))

pad <- gtable_add_padding(gt, unit(1, "cm"))
plot(pad)
plot(cbind(pad, pad))
plot(rbind(pad, pad))
}
\seealso{
Other gtable manipulation: 
\code{\link{gtable_add_cols}()},
\code{\link{gtable_add_grob}()},
\code{\link{gtable_add_rows}()},
\code{\link{gtable_add_space}},
\code{\link{gtable_filter}()}
}
\concept{gtable manipulation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable-layouts.R
\name{gtable_col}
\alias{gtable_col}
\title{Create a single column gtable}
\usage{
gtable_col(
  name,
  grobs,
  width = NULL,
  heights = NULL,
  z = NULL,
  vp = NULL,
  clip = "inherit"
)
}
\arguments{
\item{name}{a string giving the name of the table. This is used to name
the layout viewport}

\item{grobs}{a single grob or a list of grobs}

\item{width}{a unit vector giving the width of this column}

\item{heights}{a unit vector giving the height of each row}

\item{z}{a numeric vector giving the order in which the grobs should be
plotted.  Use \code{Inf} (the default) to plot above or \code{-Inf}
below all existing grobs. By default positions are on the integers,
giving plenty of room to insert new grobs between existing grobs.}

\item{vp}{a grid viewport object (or NULL).}

\item{clip}{should drawing be clipped to the specified cells
(\code{"on"}), the entire table (\code{"inherit"}), or not at all
(\code{"off"})}
}
\value{
A gtable with one column and as many rows as elements in the grobs
list.
}
\description{
This function stacks a list of grobs into a single column gtable of the given
width and heights.
}
\examples{
library(grid)
a <- rectGrob(gp = gpar(fill = "red"))
b <- circleGrob()
c <- linesGrob()
gt <- gtable_col("demo", list(a, b, c))
gt
plot(gt)
gtable_show_layout(gt)
}
\seealso{
Other gtable construction: 
\code{\link{gtable}()},
\code{\link{gtable_matrix}()},
\code{\link{gtable_row}()},
\code{\link{gtable_spacer}}
}
\concept{gtable construction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trim.R
\name{gtable_trim}
\alias{gtable_trim}
\title{Trim off empty cells.}
\usage{
gtable_trim(x)
}
\arguments{
\item{x}{a gtable object}
}
\value{
A gtable object
}
\description{
This function detects rows and columns that does not contain any grobs and
removes thewm from the gtable. If the rows and/or columns removed had a
non-zero height/width the relative layout of the gtable may change.
}
\examples{
library(grid)
rect <- rectGrob(gp = gpar(fill = "black"))
base <- gtable(unit(c(2, 2, 2), "cm"), unit(c(2, 2, 2), "cm"))

center <- gtable_add_grob(base, rect, 2, 2)
plot(center)
plot(gtable_trim(center))

col <- gtable_add_grob(base, rect, 1, 2, 3, 2)
plot(col)
plot(gtable_trim(col))

row <- gtable_add_grob(base, rect, 2, 1, 2, 3)
plot(row)
plot(gtable_trim(row))
}
\concept{gtable modification}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable-layouts.R
\name{gtable_row}
\alias{gtable_row}
\title{Create a single row gtable.}
\usage{
gtable_row(
  name,
  grobs,
  height = NULL,
  widths = NULL,
  z = NULL,
  vp = NULL,
  clip = "inherit"
)
}
\arguments{
\item{name}{a string giving the name of the table. This is used to name
the layout viewport}

\item{grobs}{a single grob or a list of grobs}

\item{height}{a unit vector giving the height of this row}

\item{widths}{a unit vector giving the width of each column}

\item{z}{a numeric vector giving the order in which the grobs should be
plotted.  Use \code{Inf} (the default) to plot above or \code{-Inf}
below all existing grobs. By default positions are on the integers,
giving plenty of room to insert new grobs between existing grobs.}

\item{vp}{a grid viewport object (or NULL).}

\item{clip}{should drawing be clipped to the specified cells
(\code{"on"}), the entire table (\code{"inherit"}), or not at all
(\code{"off"})}
}
\value{
A gtable with a single row and the same number of columns as
elements in the grobs list
}
\description{
This function puts grobs in a list side-by-side in a single-row gtable from
left to right witrh the given widths and height.
}
\examples{
library(grid)
a <- rectGrob(gp = gpar(fill = "red"))
b <- circleGrob()
c <- linesGrob()
gt <- gtable_row("demo", list(a, b, c))
gt
plot(gt)
gtable_show_layout(gt)
}
\seealso{
Other gtable construction: 
\code{\link{gtable}()},
\code{\link{gtable_col}()},
\code{\link{gtable_matrix}()},
\code{\link{gtable_spacer}}
}
\concept{gtable construction}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gtable-layouts.R
\name{gtable_matrix}
\alias{gtable_matrix}
\title{Create a gtable from a matrix of grobs.}
\usage{
gtable_matrix(
  name,
  grobs,
  widths = NULL,
  heights = NULL,
  z = NULL,
  respect = FALSE,
  clip = "on",
  vp = NULL
)
}
\arguments{
\item{name}{a string giving the name of the table. This is used to name
the layout viewport}

\item{grobs}{a single grob or a list of grobs}

\item{widths}{a unit vector giving the width of each column}

\item{heights}{a unit vector giving the height of each row}

\item{z}{a numeric matrix of the same dimensions as \code{grobs},
specifying the order that the grobs are drawn.}

\item{respect}{a logical vector of length 1: should the aspect ratio of
height and width specified in null units be respected.  See
\code{\link[=grid.layout]{grid.layout()}} for more details}

\item{clip}{should drawing be clipped to the specified cells
(\code{"on"}), the entire table (\code{"inherit"}), or not at all
(\code{"off"})}

\item{vp}{a grid viewport object (or NULL).}
}
\value{
A gtable of the same dimensions as the grobs matrix.
}
\description{
This function takes a matrix of grobs and create a gtable matching with the
grobs in the same position as they were in the matrix, with the given heights
and widths.
}
\examples{
library(grid)
a <- rectGrob(gp = gpar(fill = "red"))
b <- circleGrob()
c <- linesGrob()

row <- matrix(list(a, b, c), nrow = 1)
col <- matrix(list(a, b, c), ncol = 1)
mat <- matrix(list(a, b, c, nullGrob()), nrow = 2)

gtable_matrix("demo", row, unit(c(1, 1, 1), "null"), unit(1, "null"))
gtable_matrix("demo", col, unit(1, "null"), unit(c(1, 1, 1), "null"))
gtable_matrix("demo", mat, unit(c(1, 1), "null"), unit(c(1, 1), "null"))

# Can specify z ordering
z <- matrix(c(3, 1, 2, 4), nrow = 2)
gtable_matrix("demo", mat, unit(c(1, 1), "null"), unit(c(1, 1), "null"), z = z)
}
\seealso{
Other gtable construction: 
\code{\link{gtable}()},
\code{\link{gtable_col}()},
\code{\link{gtable_row}()},
\code{\link{gtable_spacer}}
}
\concept{gtable construction}
